/***
* Copyright (C) 1999 Gary Pennington, Robert W. Watson
* 
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public
* License along with this program; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The authors can be contacted via email at Gary.Pennington@uk.sun.com
* and Robert.W.Watson@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the program `jProf' (a program for profiling Java programs) written
* by Gary Pennington and Rob Watson.
***/
/***
*
*  Description
*
*  SCCS: jProfParser.java @(#)jProfParser.java	1.1 06/09/99 
*
***/
package jProf;

import java.io.*;
import java.util.*;

import org.w3c.dom.*;
import org.xml.sax.*;
import org.xml.sax.helpers.ParserFactory;

import com.sun.xml.parser.Resolver;


public class jProfParser
{
    public static void main(String argv[])
    {
  if (argv.length != 1) {
      System.err.println ("Usage: cmd filename");
      System.exit (1);
  }
  try
      {
    jProfParser myParser=new jProfParser();
    myParser.parse(argv[0]);
      }
  catch(IOException e)
      {
      }
    }
    
    static private String fileName;
    static private String fileBase;
    static private Vector chartList=new Vector();
    public static Vector getChartList()
    {
  return chartList;
    }
   
    public void parse(String name) throws IOException
    {
  fileName=name;
  InputSource  input;
  int fileBaseIndex=name.lastIndexOf(File.separatorChar);
  fileBase=name.substring(fileBaseIndex+1);

  try {
      input = Resolver.createInputSource (new File (fileName+".xml"));

      Parser    parser;

      parser = ParserFactory.makeParser ();
      parser.setDocumentHandler (new innerParser ());
      parser.setErrorHandler (new MyErrorHandler ());
      parser.parse (input);

  } catch (SAXParseException err) {
      System.out.println ("** Parsing error" 
    + ", line " + err.getLineNumber ()
    + ", uri " + err.getSystemId ());
      System.out.println("   " + err.getMessage ());
  } catch (SAXException e) {
      Exception  x = e;
      if (e.getException () != null)
    x = e.getException ();
      x.printStackTrace ();

  } catch (Throwable t) {
      t.printStackTrace ();
  }

    }

    static class MyErrorHandler extends HandlerBase
    {
  // treat validation errors as fatal
  public void error (SAXParseException e)
  throws SAXParseException
  {
      throw e;
  }

  // dump warnings too
  public void warning (SAXParseException err)
  throws SAXParseException
  {
      System.out.println ("** Warning" 
    + ", line " + err.getLineNumber ()
    + ", uri " + err.getSystemId ());
      System.out.println("   " + err.getMessage ());
  }
    }
  static class innerParser implements DocumentHandler
  {


    private Writer  out;
    private Writer  thread_out=null;
    private Writer  gc_out=null;
    private Writer      class_out=null;
    private FileOutputStream ostream=null;
    private ObjectOutputStream chart_dat=null;
    private Writer  mem_alloc=null;
    private Writer  mem_free=null;
    //private Writer  thr_use=null;
    private Writer  mthd_use=null;
    private Writer      curFile;
    private Writer      class_det=null;
    private long        testStart;
    private int         thrID=1;
    private int         gcID=1;
    private long        created,ended;
    private int         monThrID=0;
    private boolean     inWaitTable=false;
      private Vector thrVector=new Vector();
      private long testEnd;
      private String currentRank;
      private Vector invList=new Vector();


    // here are all the SAX DocumentHandler methods

    public void setDocumentLocator (Locator l)
    {
  // we'd record this if we needed to resolve relative URIs
  // in content or attributes, or wanted to give diagnostics.
    }

      public void startDocument ()
    throws SAXException
      {
    try {
        File outDir=new File(".."+File.separator+"Results"+File.separator+""+fileBase);
        outDir.mkdirs();
        out = new OutputStreamWriter (new FileOutputStream(".."+File.separator+"Results"+File.separator+""+fileBase+File.separator+fileBase+".html"));
        emit(out,"<HTML>\n");
    } catch (IOException e) {
        throw new SAXException ("I/O error", e);
    }
      }
      
      public void endDocument ()
    throws SAXException
      {
    try {
        out.write ("</BODY>\n</HTML>\n");
        out.flush ();
        out = null;
    } catch (IOException e) {
        throw new SAXException ("I/O error", e);
    }
      }

    public void startElement (String tag, AttributeList attrs)
    throws SAXException
    {
  if(0==tag.compareTo("header"))
      {
    testStart=Long.parseLong(attrs.getValue("start"));
    emit(out,"<HEAD>\n<TITLE>jProf (version "+attrs.getValue("agentVersion")+") results</TITLE>\n</HEAD>\n<BODY>\n");
    emit(out,"<H1>jProf Profiling Results</H1>");
    emit(out,"<p>jProf has collected information on your application which may be accessed using the following links. :-</p>");
    emit(out,"<HR WIDTH=\"50%\">");
    emit(out,"<p><A HREF=\"mem_alloc.html\">Thread Information</A></p>");
    emit(out,"<p><A HREF=\"mem_free.html\">Garbage Collection</A></p>");
    //emit(out,"<p><A HREF=\"thr_use.html\">Thread Synchronisation</A></p>");
    emit(out,"<p><A HREF=\"mthd_use.html\">Method Usage</A></p>");
    emit(out,"<p><A HREF=\"class_det.html\">Class Details</A></p>");
    try {
        mem_alloc= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+"mem_alloc.html"));
        emit(mem_alloc,"<HTML><HEAD><TITLE>Thread Details</TITLE></HEAD><BODY><H1>Thread Details</H1>\n");
        emit(mem_alloc,"<p>The following links provide information on each thread.</p>\n");

        class_det= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+"class_det.html"));
        emit(class_det,"<HTML><HEAD><TITLE>Class Details</TITLE></HEAD><BODY><H1>Class Information</H1>\n");
        emit(class_det,"<p>The following table contains information about class implementation details. The classes are listed in alphabetical order.</p>\n");
        emit(class_det,"<TABLE COLS=2 WIDTH=\"50%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER><CAPTION ALIGN=\"TOP\"> <B>Class Details</B></CAPTION>\n");
        emit(class_det,"<TR><TH>Name</TH><TH>Source File</TH></TR>\n");
        mem_free= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+""+fileBase+""+File.separator+"mem_free.html"));
        emit(mem_free,"<HTML><HEAD><TITLE>Garbage Collection</TITLE></HEAD><BODY><H1>Garbage Collection</H1>\n");
        emit(mem_free,"<p>The following table contains information about garbage collection (GC) activity which has been collected by the profiler. The GC events are listed in the order in which they occurred during the profiling (The information on memory usage is reported after the GC task has completed).</p>\n");
        emit(mem_free,"<TABLE COLS=6 WIDTH=\"50%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER><CAPTION ALIGN=\"TOP\"> <B>Garbage Collection Events</B></CAPTION>\n");
        emit(mem_free,"<TR><TH>Number</TH><TH>Elapsed Run Time (milliseconds)</TH><TH>GC Duration (milliseconds)</TH><TH>Live Object Count</TH><TH>Heap Used (bytes)</TH><TH>Heap Size (bytes)</TH></TR>\n");

        //thr_use= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+""+fileBase+""+File.separator+"thr_use.html"));
        //emit(thr_use,"<HTML><HEAD><TITLE>Thread Synchronisation</TITLE></HEAD><BODY><H1>Thread Synchronisation</H1>\n");
        //emit(thr_use,"<p>The following tables contains information about thread synchronisation activity which has been collected by the profiler. The monitor wait events are listed in the order of descending duration for each thread monitored during the profile. The target provides a link to the claas definition for the entity on which the thread was synchronised.</p>\n");

        mthd_use= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+"mthd_use.html"));
        emit(mthd_use,"<HTML><HEAD><TITLE>Method Profile</TITLE></HEAD><BODY>\n");
        ostream=new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+fileBase+".dat");
        chart_dat= new ObjectOutputStream(ostream);

    } catch (IOException e) {
        throw new SAXException ("I/O error", e);
    }
      }
  if(0==tag.compareTo("trailer"))
      {
    testEnd=Long.parseLong(attrs.getValue("end"));
    if(mem_alloc!=null)
        emit(mem_alloc,"</BODY></HTML>\n");
    if(mem_free!=null)
        emit(mem_free,"</TABLE></BODY></HTML>\n");
    //if(thr_use!=null)
        //emit(thr_use,"</BODY></HTML>\n");
    if(mthd_use!=null)
        emit(mthd_use,"</BODY></HTML>\n");
    if(class_det!=null)
        emit(class_det,"</TABLE></BODY></HTML>\n");
      }
  if(0==tag.compareTo("thread"))
      {
    if(thread_out!=null)
        {
      emit(thread_out,"</BODY></HTML>");
      try{
          thread_out.close();
      }
      catch(IOException e){
        throw new SAXException ("I/O error", e);
      }
        }
    emit(mem_alloc,"<p><A NAME=\""+attrs.getValue("thread_name")+"\" HREF=\"thread_"+String.valueOf(thrID)+".html\">"+attrs.getValue("group_name")+File.separator+attrs.getValue("parent_name")+File.separator+attrs.getValue("thread_name")+"</A></p>"+"\n");
    try {
        thread_out= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+"thread_"+String.valueOf(thrID)+".html"));
    }
    catch(IOException e){
        throw new SAXException ("I/O error", e);
    }
    emit(thread_out,"<HTML><HEAD><TITLE>Thread Details</TITLE></HEAD><BODY><H1>"+attrs.getValue("group_name")+File.separator+attrs.getValue("parent_name")+File.separator+attrs.getValue("thread_name")+"</H1>\n");
    thrID++;
    curFile=thread_out;
    handleBase(attrs);
    emit(thread_out,"<H3>Thread Details</H3><p>Monitor Waits="+attrs.getValue("monitor_wait_count")+"\n");
    emit(thread_out,"Method Calls="+attrs.getValue("method_call_count")+"</p>\n");
    thrVector.add(new lifeSpan(created, ended));
      }
  if(0==tag.compareTo("instance"))
      {
    emit(gc_out,"<H4>Object Allocation Record - <A HREF=\""+attrs.getValue("class_name")+".html\">"+attrs.getValue("class_name")+"</A> "+attrs.getValue("object_type")+"</H4>"+"\n");
    curFile=gc_out;
    handleBase(attrs);
    emit(gc_out,"<p>At this time there are "+attrs.getValue("instance_count")+" live instances of this entity, consuming "+attrs.getValue("memory_count")+" bytes.\n");
    emit(gc_out," Over the life of the profile the highest instance count was "+attrs.getValue("instance_high")+", consuming "+attrs.getValue("memory_high")+" bytes.\n");
    emit(gc_out," There were "+attrs.getValue("total_allocations")+" allocations of this entity in total, consuming "+attrs.getValue("total_memory")+" bytes.</p>\n");
    emit(gc_out,"<p>The profiler recorded that "+attrs.getValue("site_count")+" sites were used to create instances of this entity.</p>\n");
      }
  if(0==tag.compareTo("allocation_site"))
      {
    emit(gc_out,"<p>The following site was used "+attrs.getValue("count")+" times.</p>\n");
      }
  if(0==tag.compareTo("class"))
      {
    if(class_out!=null)
        {
      emit(class_out,"</BODY></HTML>");
      try{
          class_out.close();
      }
      catch(IOException e){
        throw new SAXException ("I/O error", e);
      }
        }
//     emit(class_det,"<H4>Class - <A HREF=\""+attrs.getValue("name")+".html\">"+attrs.getValue("name")+"</A> "+attrs.getValue("source_name")+"</H4>"+"\n");
    emit(class_det,"<TR><TD><A HREF=\""+attrs.getValue("name")+".html\">"+attrs.getValue("name")+"</A></TD><TD>"+attrs.getValue("source_name")+"</TD></TR>"+"\n");
    try {
        File outFile=new File(attrs.getValue("name"));
        String Path=outFile.getPath();
        int slashPos=Path.lastIndexOf(File.separatorChar);
        Path=Path.substring(0,slashPos+1);
        File outPath=new File(".."+File.separator+"Results"+File.separator+fileBase+File.separator+Path);
        outPath.mkdirs();
        class_out= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+attrs.getValue("name")+".html"));
    }
    catch(IOException e){
        throw new SAXException ("I/O error", e);
    }
    emit(class_out,"<HTML><HEAD><TITLE>Class Details</TITLE></HEAD><BODY>\n");
    curFile=class_out;
    handleBase(attrs);
    emit(class_out,"<p>This Class contains "+attrs.getValue("interface_count")+" implemented interfaces, "+attrs.getValue("method_count")+" methods,"+attrs.getValue("static_field_count")+" static fields and "+attrs.getValue("instance_field_count")+" instance fields.</p>\n");
    emit(class_out,"<TABLE COLS=3 WIDTH=\"50%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER><CAPTION ALIGN=\"TOP\"> <B>Class Implementation</B></CAPTION>\n");
    emit(class_out,"<TR><TH>Type</TH><TH>Name</TH><TH>Signature</TH></TR>\n");
        }
  if(0==tag.compareTo("method") || 0==tag.compareTo("instance_field") || 0==tag.compareTo("static_field"))
      {
    int ltPos,gtPos;
    String editedName;
    if(-1!=(ltPos=attrs.getValue("name").indexOf('<')))
        {
      StringBuffer tmpName=new StringBuffer(attrs.getValue("name"));
      tmpName.replace(ltPos,ltPos+1,"&lt;");
      gtPos=tmpName.toString().indexOf('>');
             tmpName.replace(gtPos,gtPos+1,"&gt;");
      editedName=tmpName.toString();
        }
    else
        editedName=attrs.getValue("name");
    emit(class_out,"<TR><TD>"+tag+"</TD><TD>"+editedName+"</TD><TD>"+attrs.getValue("signature")+"</TD></TR>\n");
      }
  if(0==tag.compareTo("call_trace") || 0==tag.compareTo("allocation_trace"))
      {
    if(curFile==class_out)
        emit(curFile,"</TABLE>");
    emit(curFile,"<TABLE COLS=2 WIDTH=\"50%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER>\n");
    emit(curFile,"<CAPTION ALIGN=\"TOP\"> <B>Trace</B></CAPTION>\n");
    emit(curFile,"<TR><TH>Method</TH><TH>Line</TH></TR>\n");
      }
  if(0==tag.compareTo("call_frame") || 0==tag.compareTo("allocation_frame"))
      {
    int ltPos,gtPos;
    String editedName;
    if(-1!=(ltPos=attrs.getValue("method").indexOf('<')))
        {
      StringBuffer tmpName=new StringBuffer(attrs.getValue("method"));
      tmpName.replace(ltPos,ltPos+1,"&lt;");
      gtPos=tmpName.toString().indexOf('>');
             tmpName.replace(gtPos,gtPos+1,"&gt;");
      editedName=tmpName.toString();
        }
    else
        editedName=attrs.getValue("method");
    emit(curFile,"<TR><TD>"+editedName+"</TD><TD>"+attrs.getValue("line_num")+"</TD></TR>\n");
      }
  if(0==tag.compareTo("gc"))
      {
    if(gc_out!=null)
        {
      emit(gc_out,"</BODY></HTML>");
      try{
          gc_out.close();
      }
      catch(IOException e){
        throw new SAXException ("I/O error", e);
      }
        }
    getBase(attrs);
    emit(mem_free,"<TR><TD><A HREF=\"gc_"+String.valueOf(gcID)+".html\">"+String.valueOf(gcID)+"</A></TD><TD>"+String.valueOf(created-testStart)+"</TD><TD>"+String.valueOf(ended-created)+"</TD><TD>"+attrs.getValue("used_objects")+"</TD><TD>"+attrs.getValue("used_object_space")+"</TD><TD>"+attrs.getValue("total_object_space")+"</TD></TR>"+"\n");
    curFile=mem_free;
    try {
        gc_out= new OutputStreamWriter(new FileOutputStream(".."+File.separator+"Results"+File.separator+fileBase+File.separator+"gc_"+String.valueOf(gcID)+".html"));
    }
    catch(IOException e){
        throw new SAXException ("I/O error", e);
    }
    gcID++;
      }
  if(0==tag.compareTo("wait_list"))
      {
    emit(thread_out,"<H4>Monitor waits for <A HREF=\"mem_alloc.html#"+attrs.getValue("name")+"\">"+attrs.getValue("name")+"</A></H4>\n");
      }
  if(0==tag.compareTo("monitor_wait"))
      {
    if(inWaitTable==false)
        {
      emit(thread_out,"<TABLE COLS=3 WIDTH=\"50%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER>\n");
      emit(thread_out,"<CAPTION ALIGN=\"TOP\"> <B>Monitor Waits</B></CAPTION>\n");
      emit(thread_out,"<TR><TH>Started At (milliseconds)</TH><TH>Duration (milliseconds)</TH><TH>Target Class</TH></TR>\n");
      inWaitTable=true;
        }
    getBase(attrs);
    emit(thread_out,"<TR><TD>"+String.valueOf(created-testStart)+"</TD><TD>"+attrs.getValue("duration")+"</TD><TD><A HREF=\""+attrs.getValue("target")+".html\">"+attrs.getValue("target")+"</A></TD></TR>\n");
      }
  if(0==tag.compareTo("method_use"))
      {
    emit(mthd_use,"<TABLE COLS=5 WIDTH=\"50%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER>\n");
    emit(mthd_use,"<CAPTION ALIGN=\"TOP\"> <B>Method Calls</B></CAPTION>\n");
    emit(mthd_use,"<TR><TH>Rank</TH><TH>Self (%)</TH><TH>Accumulated (%)</TH><TH>Count</TH><TH>CPU Time (milliseconds)</TH><TH>Elapsed Time (milliseconds)</TH><TH>Method</TH></TR>\n");
      }
  if(0==tag.compareTo("method_call"))
      {
    int ltPos,gtPos;
    String editedName;
    if(-1!=(ltPos=attrs.getValue("name").indexOf('<')))
        {
      StringBuffer tmpName=new StringBuffer(attrs.getValue("name"));
      tmpName.replace(ltPos,ltPos+1,"&lt;");
      gtPos=tmpName.toString().indexOf('>');
             tmpName.replace(gtPos,gtPos+1,"&gt;");
      editedName=tmpName.toString();
        }
    else
        editedName=attrs.getValue("name");
    emit(mthd_use,"<TR><TD>"+attrs.getValue("rank")+"</TD><TD>"+attrs.getValue("self")+"</TD><TD>"+attrs.getValue("total")+"</TD><TD>"+attrs.getValue("count")+"</TD><TD>"+attrs.getValue("CPUTime")+"</TD><TD>"+attrs.getValue("ElapsedTime")+"</TD><TD><A HREF=\"mthd_use.html#"+attrs.getValue("rank")+"\">"+editedName+"</A></TD></TR>\n");
    currentRank=attrs.getValue("rank");
      }
  if(0==tag.compareTo("method_invoker"))
      {
    invokerDetails myDetails=new invokerDetails(currentRank, attrs.getValue("name"), attrs.getValue("count"));
    invList.add(myDetails);
      }
    }

    public void endElement (String name)
    throws SAXException
    {
  if(0==name.compareTo("allocation_trace"))
      emit(curFile, "</TABLE>\n");
  if(0==name.compareTo("class"))
      emit(curFile, "</TABLE>\n");
  if(0==name.compareTo("call_trace"))
      {
      if(curFile!=class_out)
    emit(curFile, "</TABLE>\n");
      }
  if(0==name.compareTo("method_use"))
      {
    emit(mthd_use, "</TABLE>\n");
    String curRank="0";
    for(int i=0;i<invList.size();i++)
        {
      invokerDetails myDetails=(invokerDetails)invList.get(i);
      if(0!=myDetails.rank.compareTo(curRank))
          {
        if(0!=curRank.compareTo("0"))
        {
            emit(mthd_use, "</TABLE>\n");
            emit(mthd_use, "</BLOCKQUOTE>\n");
        }
        emit(mthd_use, "<p><A NAME=\""+myDetails.rank+"\">"+myDetails.rank+"</A>"+"</p>\n<BLOCKQUOTE>\n");
        emit(mthd_use,"<TABLE COLS=2 WIDTH=\"25%\" CELLPADDING=\"8\" CELLSPACING=\"4\" BORDER>\n");
        emit(mthd_use,"<CAPTION ALIGN=\"TOP\"> <B>Method Invokers</B></CAPTION>\n");
        emit(mthd_use,"<TR><TH>Count</TH><TH>Method</TH></TR>\n");
        curRank=myDetails.rank;
          }
          int ltPos,gtPos;
          String editedName;
          if(-1!=(ltPos=myDetails.name.indexOf('<')))
              {
            StringBuffer tmpName=new StringBuffer(myDetails.name);
            tmpName.replace(ltPos,ltPos+1,"&lt;");
            gtPos=tmpName.toString().indexOf('>');
                   tmpName.replace(gtPos,gtPos+1,"&gt;");
            editedName=tmpName.toString();
              }
          else
              editedName=myDetails.name;
      emit(mthd_use,"<TR><TD>"+myDetails.count+"</TD><TD>"+editedName+"</TD></TR>\n");
        }
        emit(mthd_use, "</TABLE>\n");
        emit(mthd_use, "</BLOCKQUOTE>\n");
      }
  if(0==name.compareTo("wait_list"))
      {
      if(inWaitTable==true)
    {
        emit(thread_out,"</TABLE>\n");
        inWaitTable=false;
    }
      }
      
  /*
  emit ("</");
  emit (name);
  emit (">");
  */
    }

    public void characters (char buf [], int offset, int len)
    throws SAXException
    {
  // NOTE:  this doesn't escape '&' and '<', but it should
  // do so else the output isn't well formed XML.  to do this
  // right, scan the buffer and write '&amp;' and '&lt' as
  // appropriate.
  try {
      if(len==1)
    return;
      emit(thread_out,"<p>\n");
      thread_out.write (buf, offset, len);
      emit(thread_out,"</p>\n");
  } catch (IOException e) {
      throw new SAXException ("I/O error", e);
  }
    }

    public void ignorableWhitespace (char buf [], int offset, int len)
    throws SAXException
    {
  // this whitespace ignorable ... so we ignore it!

  // this callback won't be used consistently by all parsers,
  // unless they read the whole DTD.  Validating parsers will
  // use it, and currently most SAX nonvalidating ones will
  // also; but nonvalidating parsers might hardly use it,
  // depending on the DTD structure.
    }

    public void processingInstruction (String target, String data)
    throws SAXException
    {
  /*
  emit ("<?");
  emit (target);
  emit (" ");
  emit (data);
  emit ("?>");
  */
    }

    // helpers ... wrap I/O exceptions in SAX exceptions, to
    // suit handler signature requirements
    private void emit (Writer tgt, String s)
    throws SAXException
    {
  try {
      tgt.write (s);
      tgt.flush ();
  } catch (IOException e) {
      throw new SAXException ("I/O error", e);
  }
    }
    private void handleBase(AttributeList attrs) throws org.xml.sax.SAXException
    {
  getBase(attrs);
  emit(curFile,"<p>First accessed at "+String.valueOf(created-testStart)+" milliseconds into the run, this entity had a lifespan of "+String.valueOf(ended-created)+" milliseconds</p>\n");
    }
    private void getBase(AttributeList attrs)
    {
  created=Long.parseLong(attrs.getValue("created"));
  ended=Long.parseLong(attrs.getValue("ended"));
    }
  }
}

class lifeSpan
{
    long created,ended;
    public lifeSpan(long created, long ended)
  {
      this.created=created;
      this.ended=ended;
  }
}

class longTuple
{
    private int first;
    private int second;
    public longTuple(int first, int second)
    {
  this.first=first;
  this.second=second;
    }
    public int getFirst()
    {
  return first;
    }
    public int getSecond()
    {
  return second;
    }
}

class invokerDetails
{
    public String rank;
    public String name;
    public String count;
    public invokerDetails(String rank, String name, String count)
    {
  this.rank=rank;
  this.name=name;
  this.count=count;
    }
}
