/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  A basic hash library used in the jProf profiler library. I wrote it to try
*  and avoid having to deal with STL type-issues between the Solaris compilers
*  and Microsoft's offering. Maybe one day, I'll switch to using the STL.
*
*  SCCS: hash.h @(#)hash.h	1.1 05/28/99 
*
***/

#ifndef HASH_INCLUDE
#define HASH_INCLUDE

#ifndef BOOL
typedef int	BOOL;
#endif

#ifndef SUCCESS
#define SUCCESS (BOOL)0
#endif

#ifndef FAIL
#define FAIL (BOOL)1
#endif

#ifndef FALSE
#define FALSE (BOOL)0
#endif

#ifndef TRUE
#define TRUE (BOOL)1
#endif
typedef void	(*memPtr)(void*);
#ifdef WIN32
#define EXPORT __declspec(dllexport)
#else
#define EXPORT
#endif

class EXPORT chainMember
{
private:
	chainMember*	m_pNext;
	chainMember*	m_pPrev;
	void*			m_key;
	void*			m_value;
  chainMember(const chainMember& rhs);
  chainMember& operator=(const chainMember& rhs);
public:
	chainMember(void *key, void* value, chainMember* pNext, chainMember* pPrev);
	~chainMember();
	void unlink(void);

	chainMember* getNext(void) const
	{
		return m_pNext;
	}
	chainMember* getPrev(void) const
	{
		return m_pPrev;
	}
	void setNext(chainMember* pNext)
	{
		m_pNext=pNext;
	}
	void setPrev(chainMember* pPrev)
	{
		m_pPrev=pPrev;
	}
	void* getKey(void) const
	{
		return m_key;
	}
	void* getValue(void) const
	{
		return m_value;
	}
	void setKey(void* newKey)
	{
		m_key=newKey;
	}
};

class EXPORT hashMember
{
private:
	chainMember*	m_pData;
  hashMember(const hashMember& rhs);
public:
  virtual hashMember& operator=(const hashMember& rhs);
	hashMember();
	~hashMember();
	BOOL addEntry(void* key, void* value);
	BOOL addEntry(chainMember* link, void* newKey);
	BOOL delEntry(void* key, BOOL bDelete=TRUE);
	chainMember* getEntry(void* key) const;
	void* getValue(void* key) const;
	void* getKey(void* key) const;
	void dump(void) const;
  unsigned long getCount(void) const;
	friend class hashIterator;
};

class EXPORT hash
{
private:
	unsigned long	m_keyCount;
	memPtr	m_memFn;
	unsigned long	m_maxKeys;
	hashMember*	m_pHashArray;
	unsigned long	m_Modulo;
	chainMember* getEntry(void* key) const;
	BOOL insert(chainMember* link, void* newKey);
	BOOL isPrime(unsigned long number);
public:
	hash(unsigned long size=10, void (*memFn)(void*)=NULL);
	hash(const hash& rhs);
  virtual hash& operator=(const hash& rhs);
	~hash();
	BOOL insert(void* key, void* value);
	void* getValue(void* key) const;
	void* getKey(void* key) const;
	BOOL remove(void* key, BOOL bDelete=TRUE);
	BOOL resize(unsigned long size);
	void dumpChains(void) const;
	unsigned long	getCount(void) const
	{
		return m_keyCount;
	}
	BOOL updateKey(void* oldKey, void* newKey);
	void empty(void);
#ifdef HASH_DEBUG
  unsigned long dbgCount(void) const;
  BOOL checkIntegrity(void)
  {
    if(dbgCount()==getCount())
      return TRUE;
    else
      return FALSE;
  }
#endif
	friend class hashIterator;
};

class EXPORT hashIterator
{
private:
	const hash&	m_Hash;
	unsigned long	m_CurrentHash;
	chainMember*	m_CurrentKey;
#ifdef HASH_DEBUG
  unsigned long m_MembersReturned;
  unsigned long m_OrigCount;
#endif
  hashIterator(const hashIterator& rhs);
  hashIterator& operator=(const hashIterator& rhs);
public:
	hashIterator(const hash& theHash);
	chainMember* getEntry();
	void* getKey(void);
	void* getValue(void);
	void reset(void);
};

#endif
