/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  Simple program for testing the hash library.
*
*  SCCS: hashTest.cpp @(#)hashTest.cpp	1.1 05/28/99 
*
***/

#include <stdlib.h>
#include <iostream.h>
#include <sys/time.h>
#include "hash.h"


int main(int argc, char** argv)
{
	if(argc!=2)
	{
		cout<<"usage: "<<argv[0]<<" <size>"<<endl;
		exit(-1);
	}
	const unsigned long testSize=strtoul(argv[1],NULL,0);
	//int testArray[testSize];
	if(testSize==0)
		exit(-1);
	int* testArray=new int[testSize];
	hash myHash(testSize);
	hrtime_t start, end;

	hashIterator myIterator(myHash);
	unsigned long j=0;
    for(int* pInt=(int*)myIterator.getValue();pInt!=NULL;pInt=(int*)myIterator.getValue(),j++)
		cout<<pInt<<endl;

	start = gethrtime();
	for(int i=0;i<testSize;i++)
	{
		testArray[i]=i;
		myHash.insert((void*)i,(void*)testArray[i]);
	}
	end = gethrtime();
	cout<<"Average Insert time = "<<(end - start)/testSize<<" nsec\n";
/*
	hashIterator myIterator(myHash);
	unsigned long j=0;
    for(int* pInt=(int*)myIterator.getValue();j<myHash.getCount();pInt=(int*)myIterator.getValue(),j++)
		cout<<pInt<<endl;
*/
	start = gethrtime();
#ifdef WIN32
	for(i=0;i<testSize;i++)
#else
	for(int i=0;i<testSize;i++)
#endif
	{
		//printf("Searching, key=%d value=%d\n",i, myHash.getValue(i));
		myHash.getValue((void*)i);
	}
	end = gethrtime();
	cout<<"Average Search time = "<<(end - start)/testSize<<" nsec\n";

	myHash.dumpChains();
	start = gethrtime();
#ifdef WIN32
	for(i=0;i<testSize;i++)
#else
	for(int i=0;i<testSize;i++)
#endif
	{
		//printf("Searching, key=%d value=%d\n",i, myHash.getValue(i));
		myHash.remove((void*)i);
/*
		if(SUCCESS==myHash.remove(i))
			printf("Remove succeeeded for %d\n",i);
		else
			printf("Remove failed for %d\n",i);
*/
	}
	end = gethrtime();
	cout<<"Average Delete time = "<<(end - start)/testSize<<" nsec\n";

	myHash.dumpChains();
	delete[] testArray;
}
