/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  See header file.
*
*  SCCS: jBaseRecord.cpp @(#)jBaseRecord.cpp	1.1 05/28/99 
*
***/

#include <fstream.h>  //Needed for definitions of streams etc...
#include <jvmpi.h>    //Needed for definitions of jobjectID etc...
#include <string.h>    //Needed for definitions of jobjectID etc...
#include "utility.h"  //Needed for time utilities, 
#include "hash.h"
#include "jBaseRecord.h"
#include "jInstanceRecord.h"
#include "jClassRecord.h"
#include "jGCRecord.h"
#include "jMethodEvent.h"
#include "jMethodInvokedBy.h"
#include "jMethodRecord.h"
#include "jThreadRecord.h"
#include "jProf.h"


jBaseRecord::jBaseRecord(): m_Ended(0), m_pTrace(NULL), m_Env(NULL)
{
  GET_TIME(m_Created);
}

jBaseRecord::jBaseRecord(const JVMPI_Event* event): m_Ended(0), m_pTrace(NULL), m_Env(event->env_id)
{
  GET_TIME(m_Created);
}

jBaseRecord::~jBaseRecord()
{
  if(m_pTrace!=NULL)
  {
    delete[] m_pTrace->frames;
    delete m_pTrace;
  }
}

/***
*
*  Description
*  Usually a jBaseRecord is timestamped when it is created, however sometimes
*  arrays of objects are created,when this happens we want to set the
*  creation time at the point of use. This function allows us to do this. 
*
***/
void jBaseRecord::setStartDetails(void)
{
  GET_TIME(m_Created);
}

void jBaseRecord::setEndDetails(JVMPI_Event* event)
{
  GET_TIME(m_Ended);
}

void jBaseRecord::getCurrentTime(unsigned long& time) const
{
  GET_TIME(time);
}

ostream& operator<<(ostream& os, jBaseRecord& jBR )
{
  unsigned long t;

  os<<"created=\""<<jBR.m_Created<<"\"";
  if(jBR.m_Ended==0)
  {
    jBR.getCurrentTime(t);
    os<<" ended=\""<<t;
  }
  else
    os<<" ended=\""<<jBR.m_Ended;
        return os<<"\" environment=\""<<jBR.m_Env<<"\"";
}
ostream& jBaseRecord::dumpTrace(ostream& os)
{
  //Dump the trace details
  if(getTrace()!=NULL)
  {
    if(getTrace()->num_frames!=0)
    {
      os<<"<call_trace>"<<endl;
      for(int i=0;i<getTrace()->num_frames;i++)
      {
        os<<"<call_frame ";
        const JVMPI_Method* pMthd=theProfiler()->getMethodByID(getTrace()->frames[i].method_id);
        jClassRecord* pCla=theProfiler()->getClassByMethodID(getTrace()->frames[i].method_id);
        //It is possible for method names to contain the < and > characters
        //Some processing is required here to replace them since they are
        //reserved characters in XML.
        char*pBuf=NULL;
        if(pMthd!=NULL)
        pBuf=getXMLSafeName(pMthd->method_name);
        if(pMthd==NULL)
        {
          if(pCla==NULL)
            os<<"method=\""<<"UNIDENTIFIED"<<"."<<"UNIDENTIFIED";
          else
          {
            if(pCla->getName())
              os<<"method=\""<<pCla->getName()<<"."<<"UNIDENTIFIED";
            else
              os<<"method=\"UNAVAILABLE."<<"UNIDENTIFIED";
          }
        }
        else
        {
          if(pCla==NULL)
            os<<"method=\""<<"UNIDENTIFIED"<<"."<<pBuf;
          else
          {
            if(pCla->getName())
              os<<"method=\""<<pCla->getName()<<"."<<pBuf;
            else
              os<<"method=\"UNAVAILABLE."<<pBuf;
          }
        }
        os<<"\" line_num=\""<<getTrace()->frames[i].lineno<<"\"/>"<<endl;
        if(pBuf!=NULL)
          delete[] pBuf;
      }
      os<<"</call_trace>"<<endl;
    }
  }
  return os;
}

jBaseRecord::jBaseRecord(const jBaseRecord& rhs)
{
  m_Created=rhs.m_Created;
  m_Ended=rhs.m_Ended;
  m_Env=rhs.m_Env;
  //Internalise the trace by copying
  if(rhs.m_pTrace!=NULL)
  {
    m_pTrace=new JVMPI_CallTrace;
    memcpy(m_pTrace,rhs.m_pTrace,sizeof(JVMPI_CallTrace));
    if(m_pTrace->num_frames>0)  //Save memory for no-frame traces, important to set frames==NULL for delete
    {
    m_pTrace->frames=new JVMPI_CallFrame[4];
    memcpy(m_pTrace->frames,rhs.m_pTrace->frames,sizeof(JVMPI_CallFrame)*4);
    }
    else
      m_pTrace->frames=NULL;
  }
  else
    m_pTrace=NULL;
}
      
jBaseRecord& jBaseRecord::operator=(const jBaseRecord& rhs)
{
  if(this==&rhs)
    return *this;

  m_Created=rhs.m_Created;
  m_Ended=rhs.m_Ended;
  m_Env=rhs.m_Env;
  if(m_pTrace)
  {
    delete[]   m_pTrace->frames;
    delete m_pTrace;
  }
  m_pTrace=rhs.m_pTrace;
  return *this;
}
