/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  This class implements base services for classes responsible for capturing
*  and logging JVMPI events. Most classes inherit core functionality from
*  here.
*
*  SCCS: jBaseRecord.h @(#)jBaseRecord.h	1.1 05/28/99 
*
***/

class jBaseRecord
{
private:
  unsigned long    m_Created;  //Creation timestamp
  unsigned long    m_Ended;    //Ended timestamp
  JVMPI_CallTrace*  m_pTrace;  //Call trace structure
  const JNIEnv*    m_Env;      //Environment
public:
  jBaseRecord();    //Array Constructor
  jBaseRecord(const JVMPI_Event* event);  //Main Constructor
  jBaseRecord(const jBaseRecord& rhs);    //Copy Constructor
  virtual ~jBaseRecord();  //Destructor
  virtual void setStartDetails(void);  //Set start details
  virtual void  setEndDetails(JVMPI_Event* event=NULL);  //Set end details
  BOOL  isAlive(void) const  //Check if the object has ended
  {
    return !m_Ended;
  }
  void getCurrentTime(unsigned long& time) const;  //Convenience time function
  friend ostream& operator<< (ostream& os, jBaseRecord& jBR );  //Output
  ostream& dumpTrace(ostream& os);  //Dump any associated trace records
  const JNIEnv* getEnv(void) const  //Get the embedded env pointer
  {
    return m_Env;
  }
  virtual jBaseRecord& operator=(const jBaseRecord& rhs);  //Assignment operator
  void setTrace(JVMPI_CallTrace* trace)  //Associate a trace
  {
    m_pTrace=trace;
  }
  const JVMPI_CallTrace* getTrace(void) const  //Get a trace
  {
    return m_pTrace;
  }
  const unsigned long getCreated(void) const  //Get creation timestamp
  {
    return m_Created;
  }
  const unsigned long getEnded(void) const  //Get ended timestamp
  {
    return m_Ended;
  }
  const unsigned long getDuration(void) const  //Get duration of this event
  {
    unsigned long t;
    if(isAlive())
    {
      getCurrentTime(t);
      return t-m_Created;
    }
    else
      return m_Ended-m_Created;
  }
  virtual BOOL operator<(const jBaseRecord& rhs) const  //Operator less than
  {
      return (getCreated()<rhs.getCreated());
  }
  virtual BOOL operator>(const jBaseRecord& rhs) const  //Operator greater than
  {
      return (getCreated()>rhs.getCreated());
  }
};
