/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  See header file.
*
*  SCCS: jClassRecord.cpp @(#)jClassRecord.cpp	1.2 06/08/99 
*
***/

#include <fstream.h>  //Needed for definitions of streams etc...
#include <jvmpi.h>    //Needed for definitions of jobjectID etc...
#ifndef WIN32
#include <string.h>
#include <strings.h>
#endif
#include "utility.h"  //Needed for time utilities, 
#include "hash.h"
#include "jBaseRecord.h"
#include "jInstanceRecord.h"
#include "jClassRecord.h"

jClassRecord::jClassRecord(JVMPI_Event* event) : jBaseRecord(event), m_ID(event->u.class_load.class_id), m_pInstance(NULL)
{
  int strLen=0;
  
  if(event->u.class_load.class_name)
  {
		m_Name=getXMLSafeName(event->u.class_load.class_name);
//     strLen=strlen(event->u.class_load.class_name);
//     m_Name=new char[strLen+1];
//     strcpy(m_Name,event->u.class_load.class_name);
  }
  else
    m_Name=NULL;
  
  if(event->u.class_load.source_name)
  {
		m_SourceName=getXMLSafeName(event->u.class_load.source_name);
//     strLen=strlen(event->u.class_load.source_name);
//     m_SourceName=new char[strLen+1];
//     strcpy(m_SourceName,event->u.class_load.source_name);
  }
  else
    m_SourceName=NULL;
  
  m_NumInterfaces=event->u.class_load.num_interfaces;
  m_NumMethods=event->u.class_load.num_methods;
  //You don't seem to need to copy the data
  //members of the JVMPI_Method structure, they remain valid
  
  JVMPI_Method* pMyMethod=new JVMPI_Method[m_NumMethods];
  memcpy(pMyMethod,event->u.class_load.methods,m_NumMethods*sizeof(JVMPI_Method));
  m_Methods=pMyMethod;
  m_NumStaticFields=event->u.class_load.num_static_fields;
  JVMPI_Field* pMyStatic=new JVMPI_Field[m_NumStaticFields];
  memcpy(pMyStatic,event->u.class_load.statics,m_NumStaticFields*sizeof(JVMPI_Field));
  m_StaticFields=pMyStatic;
  m_NumInstFields=event->u.class_load.num_instance_fields;
  JVMPI_Field* pMyInst=new JVMPI_Field[m_NumInstFields];
  memcpy(pMyInst,event->u.class_load.instances,m_NumInstFields*sizeof(JVMPI_Field));
  m_InstFields=pMyInst;
}

jClassRecord::jClassRecord(JVMPI_Event* event, jint arrayType) : jBaseRecord(event), m_pInstance(NULL)
{
  m_ID=(jobjectID)arrayType;
  switch(arrayType)
  {
    case JVMPI_NORMAL_OBJECT:
      m_Name=new char[strlen("ArrayType - Object")+1];
      strcpy(m_Name,"ArrayType - Object");
      break;
    case JVMPI_CLASS:
      m_Name=new char[strlen("ArrayType - Class")+1];
      strcpy(m_Name,"ArrayType - Class");
      break;
    case JVMPI_BOOLEAN:
      m_Name=new char[strlen("ArrayType - Boolean")+1];
      strcpy(m_Name,"ArrayType - Boolean");
      break;
    case JVMPI_BYTE:
      m_Name=new char[strlen("ArrayType - Byte")+1];
      strcpy(m_Name,"ArrayType - Byte");
      break;
    case JVMPI_CHAR:
      m_Name=new char[strlen("ArrayType - Char")+1];
      strcpy(m_Name,"ArrayType - Char");
      break;
    case JVMPI_SHORT:
      m_Name=new char[strlen("ArrayType - Short")+1];
      strcpy(m_Name,"ArrayType - Short");
      break;
    case JVMPI_INT:
      m_Name=new char[strlen("ArrayType - Int")+1];
      strcpy(m_Name,"ArrayType - Int");
      break;
    case JVMPI_LONG:
      m_Name=new char[strlen("ArrayType - Long")+1];
      strcpy(m_Name,"ArrayType - Long");
      break;
    case JVMPI_FLOAT:
      m_Name=new char[strlen("ArrayType - Float")+1];
      strcpy(m_Name,"ArrayType - Float");
      break;
    case JVMPI_DOUBLE:
      m_Name=new char[strlen("ArrayType - Double")+1];
      strcpy(m_Name,"ArrayType - Double");
      break;
    default:
      m_Name=new char[strlen("ArrayType - Unknown")+1];
      strcpy(m_Name,"ArrayType - Unknown");
      break;
  }
  m_SourceName=new char[strlen("Not Applicable")+1];
  strcpy(m_SourceName,"Not Applicable");
  m_NumInterfaces=0;
  m_NumMethods=0;
  m_Methods=NULL;
  m_NumStaticFields=0;
  m_StaticFields=NULL;
  m_NumInstFields=0;
  m_InstFields=NULL;
}

jClassRecord::~jClassRecord()
{
  delete[] m_SourceName;
  delete[] m_Name;
  delete[] m_Methods;
  delete[] m_StaticFields;
  delete[] m_InstFields;
  delete m_pInstance;
}

jClassRecord& jClassRecord::operator=(jClassRecord& rhs)
{
  if(this==&rhs)
    return *this;
  
  jBaseRecord::operator=(rhs);
  
  m_ID=rhs.m_ID;
  delete[] m_SourceName;
  m_SourceName=new char[sizeof(rhs.m_SourceName)];
  strcpy(m_SourceName,rhs.m_SourceName);
  delete[] m_Name;
  m_Name=new char[sizeof(rhs.m_Name)];
  strcpy(m_Name,rhs.m_Name);
  m_NumInterfaces=rhs.m_NumInterfaces;
  m_NumMethods=rhs.m_NumMethods;
  delete[] m_Methods;
  m_Methods=new JVMPI_Method[m_NumMethods];
  memcpy(m_Methods,rhs.m_Methods,m_NumMethods*sizeof(JVMPI_Method));
  m_NumStaticFields=rhs.m_NumStaticFields;
  delete[] m_StaticFields;
  m_StaticFields=new JVMPI_Field[m_NumStaticFields];
  memcpy(m_StaticFields,rhs.m_StaticFields,m_NumStaticFields*sizeof(JVMPI_Field));
  m_NumInstFields=rhs.m_NumInstFields;
  delete[] m_InstFields;
  m_InstFields=new JVMPI_Field[m_NumInstFields];
  memcpy(m_InstFields,rhs.m_InstFields,m_NumInstFields*sizeof(JVMPI_Field));
  delete m_pInstance;
  if(rhs.m_pInstance!=NULL)
    m_pInstance=new jInstanceRecord(*rhs.m_pInstance);
  else
    m_pInstance=NULL;
  
  return *this;
}

ostream& operator<< ( ostream& os, jClassRecord& jCR )
{
  jBaseRecord& jBR=jCR;
  os<<"<class "<<jBR;
  os<<" id=\""<<jCR.m_ID<<"\"";
  os<<" name=\"";
  if(jCR.m_Name)
    os<<jCR.m_Name;
  else
    os<<"none";
  os<<"\" source_name=\"";
  if(jCR.m_SourceName)
    os<<jCR.m_SourceName;
  else
    os<<"none";
  os<<"\" interface_count=\""<<jCR.m_NumInterfaces;
  os<<"\" method_count=\""<<jCR.m_NumMethods;
  os<<"\" static_field_count=\""<<jCR.m_NumStaticFields;
  os<<"\" instance_field_count=\""<<jCR.m_NumInstFields<<"\">"<<endl;
  JVMPI_Method* pMethod=jCR.m_Methods;
  for(int i=0;i<jCR.m_NumMethods;i++)
  {
    //It is possible for method names to contain the < and > characters
    //Some processing is required here to replace them since they are
    //reserved characters in XML.
    char* pBuf=getXMLSafeName(pMethod->method_name);
    os<<"<method name=\""<<pBuf;
    delete[] pBuf;
    
    os<<"\" signature=\""<<pMethod->method_signature;
    os<<"\" method_id=\""<<pMethod->method_id;
    os<<"\"/>"<<endl;
    pMethod++;
  }
  JVMPI_Field* pStatic=jCR.m_StaticFields;
#ifdef WIN32
  for(i=0;i<jCR.m_NumStaticFields;i++)
#else
  for(int i=0;i<jCR.m_NumStaticFields;i++)
#endif
  {
  	os<<"<static_field name=\""<<pStatic->field_name;
  	os<<"\" signature=\""<<pStatic->field_signature;
  	os<<"\"/>"<<endl;
  	pStatic++;
  }
  JVMPI_Field* pInst=jCR.m_InstFields;
#ifdef WIN32
  for(i=0;i<jCR.m_NumInstFields;i++)
#else
  for(int i=0;i<jCR.m_NumInstFields;i++)
#endif
  {
  	os<<"<instance_field name=\""<<pInst->field_name;
  	os<<"\" signature=\""<<pInst->field_signature;
  	os<<"\"/>"<<endl;
  	pInst++;
  }
  jCR.dumpTrace(os);
  if(jCR.m_pInstance!=NULL)
    os<<*jCR.m_pInstance;
  return os<<"</class>"<<endl;
}
