/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  See header file.
*
*  SCCS: jGCRecord.cpp @(#)jGCRecord.cpp	1.3 06/09/99 
*
***/

#include <fstream.h>  //Needed for definitions of streams etc...
#include <jvmpi.h>    //Needed for definitions of jobjectID etc...
#ifndef WIN32
#include <string.h>
#endif
#include "utility.h"  //Needed for time utilities, 
#include "hash.h"
#include "jBaseRecord.h"
#include "jInstanceRecord.h"
#include "jGCRecord.h"

extern void manageInstance(void* pData);

jGCRecord::jGCRecord(JVMPI_Event* event) : jBaseRecord(event), m_ObjHash(1000, manageInstance), m_ObjFreed(0), m_bShowAllocSites(FALSE)
{
  GET_TIME(m_ID);  //Assume that the time of a GC is a unique identifier
  m_UsedObjects=m_UsedObjectSpace=m_TotalObjectSpace=0;
}

jGCRecord& jGCRecord::operator=(const jGCRecord& rhs)
{
  if(this==&rhs)
    return *this;

  jBaseRecord::operator=(rhs);

  m_ID=rhs.m_ID;
  m_UsedObjects=rhs.m_UsedObjects;
  m_UsedObjectSpace=rhs.m_UsedObjectSpace;
  m_TotalObjectSpace=rhs.m_TotalObjectSpace;
  m_ObjHash=rhs.m_ObjHash;
  m_ObjFreed=rhs.m_ObjFreed;
  m_bShowAllocSites=rhs.m_bShowAllocSites;
  
  return *this;
}

ostream& operator<<(ostream& os, jGCRecord& jGC )
{
  jBaseRecord& jBR=jGC;
  os<<"<gc "<<jBR;
  os<<" used_objects=\""<<(long)jGC.m_UsedObjects;
  os<<"\" used_object_space=\""<<(long)jGC.m_UsedObjectSpace;
  os<<"\" total_object_space=\""<<(long)jGC.m_TotalObjectSpace;
  os<<"\" objects_released=\""<<(long)jGC.m_ObjFreed<<"\">"<<endl;
  /*Sort the object instance records into descending memory used order*/
  if(jGC.m_ObjHash.getCount()>0)
  {
    jInstanceRecord**  ppObj=new jInstanceRecord*[jGC.m_ObjHash.getCount()];
    memset(ppObj,0,jGC.m_ObjHash.getCount()*sizeof(jInstanceRecord*));
    hashIterator mrIterator(jGC.m_ObjHash);
    unsigned long i=0;
    for(jInstanceRecord* pObj=(jInstanceRecord*)mrIterator.getValue();pObj!=NULL;pObj=(jInstanceRecord*)mrIterator.getValue(),i++)
    {
      ppObj[i]=pObj;
    }
    //At this stage ppObj contains a series of pointers all pointing at the original pObj records
		quicksort(ppObj,0,jGC.m_ObjHash.getCount()-1,FALSE);
		/*
    for(unsigned long k=0;k<jGC.m_ObjHash.getCount()-1;k++)
    {
      for(unsigned long j=jGC.m_ObjHash.getCount()-1;j>k;j--)
      {
        if(*(ppObj[j-1])<*(ppObj[j]))
        {
          jInstanceRecord* pTmp=ppObj[j-1];
          ppObj[j-1]=ppObj[j];
          ppObj[j]=pTmp;
        }
      }
    }
		*/
    //Now have a sorted list for the gc records
    for(unsigned long j=0;j<jGC.m_ObjHash.getCount();j++)
    {
      if(jGC.getShowAllocSites())
        ppObj[j]->setShowAllocSites(TRUE);
      os<<*(jInstanceRecord*)ppObj[j];
    }
    delete[] ppObj;
  }
  return os<<"</gc>"<<endl;
}

void jGCRecord::setEndDetails(JVMPI_Event* event)
{
  jBaseRecord::setEndDetails();
  m_UsedObjects=event->u.gc_info.used_objects;
  m_UsedObjectSpace=event->u.gc_info.used_object_space;
  m_TotalObjectSpace=event->u.gc_info.total_object_space;
}

