/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  See header file.
*
*  SCCS: jInstanceRecord.cpp @(#)jInstanceRecord.cpp	1.3 06/09/99 
*
***/

#include <fstream.h>  //Needed for definitions of streams etc...
#include <jvmpi.h>    //Needed for definitions of jobjectID etc...
#ifndef WIN32
#include <string.h>
#include <strings.h>
#endif
/*
#include "utility.h"  //Needed for time utilities, 
#include "jBaseRecord.h"
#include "hash.h"
#include "jInstanceRecord.h"
#include "jClassRecord.h"
*/
#include "utility.h"  //Needed for time utilities, 
#include "hash.h"
#include "jBaseRecord.h"
#include "jInstanceRecord.h"
#include "jClassRecord.h"
#include "jGCRecord.h"
#include "jMethodEvent.h"
#include "jMethodInvokedBy.h"
#include "jMethodRecord.h"
#include "jThreadRecord.h"
#include "jProf.h"

const unsigned long allocBlock=25;

jInstanceRecord::jInstanceRecord(const JVMPI_Event* event, const jClassRecord* pCla, JVMPI_CallTrace* pTrace) : jBaseRecord(event), m_IsArray(event->u.obj_alloc.is_array), m_pClass(pCla), m_pAllocList(NULL), m_bShowAllocSites(FALSE)
{
//  m_TotalCountAllocated=m_InstanceCount=m_InstanceHigh=1;
//  m_TotalMemoryAllocated=m_Size=m_MemoryCount=m_MemoryHigh=event->u.obj_alloc.size;
  m_AllocCount=0;
  m_TotalCountAllocated=m_InstanceCount=m_InstanceHigh=0;
  m_TotalMemoryAllocated=m_Size=m_MemoryCount=m_MemoryHigh=0;
  add(event,pTrace);
}

jInstanceRecord::~jInstanceRecord()
{
  for(unsigned long i=0;i<m_AllocCount;i++)
    delete m_pAllocList[i];
  delete[] m_pAllocList;
}

jInstanceRecord::jInstanceRecord(const jInstanceRecord& rhs) : jBaseRecord(rhs)
{
  m_IsArray=rhs.m_IsArray;
  m_InstanceCount=rhs.m_InstanceCount;
  m_InstanceHigh=rhs.m_InstanceHigh;
  m_MemoryCount=rhs.m_MemoryCount;
  m_MemoryHigh=rhs.m_MemoryHigh;
  m_pClass=rhs.m_pClass;
  m_TotalCountAllocated=rhs.m_TotalCountAllocated;
  m_TotalMemoryAllocated=rhs.m_TotalMemoryAllocated;
  m_Size=rhs.m_Size;
  m_bShowAllocSites=rhs.m_bShowAllocSites;
  //At this point, we need to copy in the allocation records from the rhs
  m_AllocCount=0;
  m_pAllocList=NULL;
  if(rhs.m_AllocCount>0)
  {
    //Allocate new pointers for holding our data, do this in blocks of allocBlock
    unsigned long newCount=(rhs.m_AllocCount/allocBlock)+1;
    m_pAllocList=new jAllocationRecord*[newCount*allocBlock];
    memset(m_pAllocList,0,sizeof(jAllocationRecord*)*(newCount*allocBlock));
    for(unsigned long i=0;i<rhs.m_AllocCount;i++)
    {
      m_pAllocList[m_AllocCount++]=new jAllocationRecord(*(rhs.m_pAllocList[i]));
    }
  }
}

jInstanceRecord& jInstanceRecord::operator+=(const jInstanceRecord& rhs)
{
  //At this point, we need to merge in the allocation records from the rhs
  if(rhs.m_AllocCount>0)
  {
    for(unsigned long i=0;i<rhs.m_AllocCount;i++)
    {
      //Try to find the allocation record in our existing list
#ifdef WIN32
      for(unsigned long j=0;j<m_AllocCount;j++)
#else
      unsigned long j;
      for(j=0;j<m_AllocCount;j++)
#endif
      {
        if((*(m_pAllocList[j]))==(*(rhs.m_pAllocList[i])))//Found one
        {
          m_pAllocList[j]->incrementCount();
          break;
        }
      }
      if(j==m_AllocCount) //ie Not Found
      {
        if(m_AllocCount%allocBlock==0)
        {
          jAllocationRecord**  pNewRecords=new jAllocationRecord*[m_AllocCount+allocBlock];
          memset(pNewRecords,0,sizeof(jAllocationRecord*)*(m_AllocCount+allocBlock));
          if(m_AllocCount>0)
          {
            memcpy(pNewRecords,m_pAllocList,m_AllocCount*sizeof(jAllocationRecord*));
            delete[] m_pAllocList;
          }
          m_pAllocList=pNewRecords;
        }
        m_pAllocList[m_AllocCount++]=new jAllocationRecord(*(rhs.m_pAllocList[i]));
      }
    }
  }
  m_InstanceCount+=rhs.m_InstanceCount;
  if(m_InstanceCount>m_InstanceHigh)
    m_InstanceHigh=m_InstanceCount;
  m_MemoryCount+=rhs.m_MemoryCount;
  if(m_MemoryCount>m_MemoryHigh)
    m_MemoryHigh=m_MemoryCount;
  m_TotalCountAllocated+=rhs.m_TotalCountAllocated;
  m_TotalMemoryAllocated+=rhs.m_TotalMemoryAllocated;
  return *this;
}

jInstanceRecord& jInstanceRecord::operator-=(const jInstanceRecord& rhs)
{
  m_InstanceCount-=rhs.m_InstanceCount;
  m_MemoryCount-=rhs.m_MemoryCount;

  return *this;
}

jInstanceRecord& jInstanceRecord::operator=(const jInstanceRecord& rhs)
{
  if(this==&rhs)
    return *this;

  jBaseRecord::operator=(rhs);
  m_IsArray=rhs.m_IsArray;
  m_InstanceCount=rhs.m_InstanceCount;
  m_InstanceHigh=rhs.m_InstanceHigh;
  m_MemoryCount=rhs.m_MemoryCount;
  m_MemoryHigh=rhs.m_MemoryHigh;
  m_pClass=rhs.m_pClass;
  m_TotalCountAllocated=rhs.m_TotalCountAllocated;
  m_TotalMemoryAllocated=rhs.m_TotalMemoryAllocated;
  m_Size=rhs.m_Size;
  m_bShowAllocSites=rhs.m_bShowAllocSites;
  //At this point, we need to copy in the allocation records from the rhs
  for(unsigned long i=0;i<m_AllocCount;i++)
    delete m_pAllocList[i];
  delete[] m_pAllocList;
  m_pAllocList=NULL;
  m_AllocCount=0;
  if(rhs.m_AllocCount>0)
  {
    //Allocate new pointers for holding our data, do this in blocks of allocBlock
    unsigned long newCount=(rhs.m_AllocCount/allocBlock)+1;
    m_pAllocList=new jAllocationRecord*[newCount*allocBlock];
    memset(m_pAllocList,0,sizeof(jAllocationRecord*)*(newCount*allocBlock));
    for(unsigned long i=0;i<rhs.m_AllocCount;i++)
    {
      m_pAllocList[m_AllocCount++]=new jAllocationRecord(*(rhs.m_pAllocList[i]));
    }
  }
  
  return *this;
}

void jInstanceRecord::add(const JVMPI_Event* event, JVMPI_CallTrace* pTrace)
{
  m_InstanceCount++;
  if(m_InstanceCount>m_InstanceHigh)
    m_InstanceHigh=m_InstanceCount;
  m_MemoryCount+=event->u.obj_alloc.size;
  if(m_MemoryCount>m_MemoryHigh)
    m_MemoryHigh=m_MemoryCount;
  m_TotalCountAllocated++;
  m_TotalMemoryAllocated+=event->u.obj_alloc.size;
  //Add an allocation trace record so that we can keep track of where this object
  //was allocated. This allows us to keep track of allocations per instance per thread
  //When a GC starts, these records will be sorted and merged to generate the GC
  //allocation report.
  //Try to find the allocation record in our existing list
  jAllocationRecord* pNewRecord=new jAllocationRecord(pTrace);
#ifdef WIN32
  for(unsigned long j=0;j<m_AllocCount;j++)
#else
  unsigned long j;
  for(j=0;j<m_AllocCount;j++)
#endif
  {
    if(*(m_pAllocList[j])==*(pNewRecord))//Found one
    {
      m_pAllocList[j]->incrementCount();
      delete pNewRecord;
      break;
    }
  }
  if(j==m_AllocCount) //ie Not Found
  {
    if(m_AllocCount%allocBlock==0)
    {
      jAllocationRecord**  pNewRecords=new jAllocationRecord*[m_AllocCount+allocBlock];
      memset(pNewRecords,0,sizeof(jAllocationRecord*)*(m_AllocCount+allocBlock));
      if(m_AllocCount>0)
      {
        memcpy(pNewRecords,m_pAllocList,m_AllocCount*sizeof(jAllocationRecord*));
        delete[] m_pAllocList;
      }
      m_pAllocList=pNewRecords;
    }
    m_pAllocList[m_AllocCount++]=pNewRecord;
  }
}

void jInstanceRecord::sub(void)
{
  m_InstanceCount--;
  m_MemoryCount-=m_Size;
}

ostream& operator<<(ostream& os, jInstanceRecord& jIR )
{
  jBaseRecord& jBR=jIR;

  os<<"<instance "<<jBR;
  os<<" instance_count=\""<<(long)jIR.m_InstanceCount;
  os<<"\" instance_high=\""<<(long)jIR.m_InstanceHigh;
  os<<"\" memory_count=\""<<(long)jIR.m_MemoryCount;
  os<<"\" memory_high=\""<<(long)jIR.m_MemoryHigh;
  os<<"\" total_allocations=\""<<(long)jIR.m_TotalCountAllocated;
  os<<"\" total_memory=\""<<(long)jIR.m_TotalMemoryAllocated;
  if(jIR.m_pClass!=NULL)
    os<<"\" class_name=\""<<jIR.m_pClass->getName();
  else
    os<<"\" class_name=\""<<"array type";
  os<<"\" object_type=\"";
  switch(jIR.m_IsArray)
  {
    case JVMPI_NORMAL_OBJECT:
      os<<"Object";
      break;
    case JVMPI_CLASS:
      os<<"Array of Objects";
      break;
    case JVMPI_BOOLEAN:
      os<<"Array of Booleans";
      break;
    case JVMPI_BYTE:
      os<<"Array of Bytes";
      break;
    case JVMPI_CHAR:
      os<<"Array of Chars";
      break;
    case JVMPI_SHORT:
      os<<"Array of Shorts";
      break;
    case JVMPI_INT:
      os<<"Array of Ints";
      break;
    case JVMPI_LONG:
      os<<"Array of Longs";
      break;
    case JVMPI_FLOAT:
      os<<"Array of Floats";
      break;
    case JVMPI_DOUBLE:
      os<<"Array of Doubles";
      break;
    default:
      os<<"Unrecognised Array Type!";
  }
  if(jIR.m_bShowAllocSites)
  {
    os<<"\" site_count=\""<<jIR.m_AllocCount<<"\">"<<endl;
    if(jIR.m_pAllocList) //Dump Allocation Traces
    {
      //Implement a sort here when resume tomorrow 17/5/99
      jAllocationRecord**  ppAlloc=new jAllocationRecord*[jIR.m_AllocCount];
      memset(ppAlloc,0,jIR.m_AllocCount*sizeof(jAllocationRecord*));
      for(unsigned long i=0;i<jIR.m_AllocCount;i++)
      {
        ppAlloc[i]=jIR.m_pAllocList[i];
      }
      //At this stage ppAlloc contains a series of pointers all pointing at the original pAlloc records
			//bubblesort replacement
			if(jIR.m_AllocCount>1)
			{
			quicksort(ppAlloc,0,jIR.m_AllocCount-1,FALSE);
			}
			/*
      for(unsigned long k=0;k<jIR.m_AllocCount-1;k++)
      {
        for(unsigned long j=jIR.m_AllocCount-1;j>k;j--)
        {
          if(*(ppAlloc[j-1])<*(ppAlloc[j]))
          {
            jAllocationRecord* pTmp=ppAlloc[j-1];
            ppAlloc[j-1]=ppAlloc[j];
            ppAlloc[j]=pTmp;
          }
        }
      }
			*/
      //Now have a sorted list for the allocation records
      for(unsigned long j=0;j<jIR.m_AllocCount && j<20;j++)
      {
        os<<*(jAllocationRecord*)ppAlloc[j];
      }
      delete[] ppAlloc;
    }
    return os<<"</instance>"<<endl;
  }
  else
    return os<<"\"/>"<<endl;
}

jAllocationRecord& jAllocationRecord::operator=(const jAllocationRecord& rhs)
{

  if(this==&rhs)
    return *this;

  m_Count=rhs.m_Count;
  if(m_pTrace)
  {
    delete[] m_pTrace->frames;  //This is set to NULL in getCallTrace if the frame_count is 0
    delete m_pTrace;
  }
  m_pTrace=new JVMPI_CallTrace;
  m_pTrace->frames= new JVMPI_CallFrame[rhs.m_pTrace->num_frames];
  memcpy(m_pTrace,rhs.m_pTrace,sizeof(JVMPI_CallTrace));
  memcpy(m_pTrace->frames,rhs.m_pTrace->frames,sizeof(JVMPI_CallFrame)*rhs.m_pTrace->num_frames);
  return *this;
}

jAllocationRecord::jAllocationRecord(const jAllocationRecord& rhs)
{
  m_Count=rhs.m_Count;
  m_pTrace=new JVMPI_CallTrace;
  memcpy(m_pTrace,rhs.m_pTrace,sizeof(JVMPI_CallTrace));
  if(rhs.m_pTrace->num_frames>0)
  {
    m_pTrace->frames= new JVMPI_CallFrame[rhs.m_pTrace->num_frames];
    memcpy(m_pTrace->frames,rhs.m_pTrace->frames,sizeof(JVMPI_CallFrame)*rhs.m_pTrace->num_frames);
  }
}

ostream& operator<<(ostream& os, jAllocationRecord& jAR )
{
  os<<"<allocation_site count=\""<<(long)jAR.m_Count<<"\">"<<endl;
  jAR.dumpTrace(os);
  return os<<"</allocation_site>"<<endl;
}

ostream& jAllocationRecord::dumpTrace(ostream& os)
{
  //Dump the trace details
  if(getTrace()!=NULL)
  {
    if(getTrace()->num_frames!=0)
    {
      os<<"<allocation_trace>"<<endl;
      for(int i=0;i<getTrace()->num_frames;i++)
      {
        os<<"<allocation_frame ";
        const JVMPI_Method* pMthd=theProfiler()->getMethodByID(getTrace()->frames[i].method_id);
        jClassRecord* pCla=theProfiler()->getClassByMethodID(getTrace()->frames[i].method_id);
        //It is possible for method names to contain the < and > characters
        //Some processing is required here to replace them since they are
        //reserved characters in XML.
        char*pBuf=NULL;
        if(pMthd!=NULL)
        pBuf=getXMLSafeName(pMthd->method_name);
        if(pMthd==NULL)
        {
          if(pCla==NULL)
            os<<"method=\""<<"UNIDENTIFIED"<<"."<<"UNIDENTIFIED";
          else
          {
            if(pCla->getName())
              os<<"method=\""<<pCla->getName()<<"."<<"UNIDENTIFIED";
            else
              os<<"method=\"UNAVAILABLE."<<"UNIDENTIFIED";
          }
        }
        else
        {
          if(pCla==NULL)
            os<<"method=\""<<"UNIDENTIFIED"<<"."<<pBuf;
          else
          {
            if(pCla->getName())
              os<<"method=\""<<pCla->getName()<<"."<<pBuf;
            else
              os<<"method=\"UNAVAILABLE."<<pBuf;
          }
        }
        os<<"\" line_num=\""<<getTrace()->frames[i].lineno<<"\"/>"<<endl;
        if(pBuf!=NULL)
          delete[] pBuf;
      }
      os<<"</allocation_trace>"<<endl;
    }
  }
  return os;
}


