/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  This class implements base services for recording allocation site data.
*  The class keeps a trace which show it's allocation site and a count which
*  is the number of times that this allocation site has been used.
*
*  SCCS: jInstanceRecord.h @(#)jInstanceRecord.h	1.1 05/28/99 
*
***/

//Forward Declaration
class jClassRecord;

class jAllocationRecord
{
 private:
  unsigned long m_Count;      //Invocation count
  JVMPI_CallTrace*  m_pTrace; //Trace for this allocation reference
 public:
  jAllocationRecord(JVMPI_CallTrace* pTrace) : m_Count(1), m_pTrace(pTrace)//Main Constructor
  {
  }
  ~jAllocationRecord()
  {
    delete[] m_pTrace->frames;  //This is set to NULL in getCallTrace if the frame_count is 0
    delete m_pTrace;
  }
  jAllocationRecord(const jAllocationRecord& rhs);  //Copy Constructor
  friend ostream& operator<< ( ostream& os, jAllocationRecord& jMIB );  //Output
  ostream& dumpTrace(ostream& os);  //Dump any associated trace records
  unsigned long getCount(void) const  //Get Invocation Count
  {
    return m_Count;
  }
  void incrementCount(void)  //Increment Invocation Count
  {
    m_Count++;
  }
  const JVMPI_CallTrace* getTrace(void) const  //Get a trace
  {
    return m_pTrace;
  }
  virtual BOOL operator==(const jAllocationRecord& rhs) const  //Equality
  {
    BOOL bRet=TRUE;

    for(int i=0;i<m_pTrace->num_frames && i<rhs.m_pTrace->num_frames;i++)
    {
      if(m_pTrace->frames[i].method_id!=rhs.m_pTrace->frames[i].method_id)
      {
        bRet=FALSE;
        break;
      }
      if(m_pTrace->frames[i].lineno!=rhs.m_pTrace->frames[i].lineno)
      {
        bRet=FALSE;
        break;
      }
    }
    return bRet;
  }
  virtual jAllocationRecord& operator=(const jAllocationRecord& rhs);  //Assignment
  //virtual jAllocationRecord& operator+=(jAllocationRecord& rhs); //Add&Assign
  virtual BOOL operator<(const jAllocationRecord& rhs) const  //Operator less than
  {
      return (m_Count<rhs.m_Count);
  }
  virtual BOOL operator>(const jAllocationRecord& rhs) const  //Operator greater than
  {
      return (m_Count>rhs.m_Count);
  }
};

/***
*
*  Description
*  This class implements base services for recording Instance event data
*  supplied by the JVMPI.
*  Those funtions which are not virtual are invariant, ie it is not valid
*  for a derived class to re-define them.
*
*  Modification Log
*
*  Version    Date  Description
*  1.0      8/5/99  Initial Release
*
***/

class jInstanceRecord : public jBaseRecord
{
private:
  jint  m_IsArray;                  //Array indicator, supplied by JVMPI
  const jClassRecord*  m_pClass;    //Class owner
  jlong m_InstanceCount;            //Count of allocated instances
  jlong m_InstanceHigh;             //High count of allocated instances
  jlong m_MemoryCount;              //Count of allocated memory(bytes)
  jlong m_MemoryHigh;               //High count of allocated memory(bytes)
  jlong m_TotalCountAllocated;      //Total count of allocated instances
  jlong m_TotalMemoryAllocated;     //Total count of allocated memory(bytes)
  jlong  m_Size;                    //Size of this object
  jAllocationRecord** m_pAllocList;  //List of allocations
  unsigned long m_AllocCount;
  BOOL  m_bShowAllocSites;
public:
  jInstanceRecord(const JVMPI_Event* event, const jClassRecord* pCla, JVMPI_CallTrace* pTrace);  //Main Constructor
  virtual ~jInstanceRecord();  //Destructor
  friend ostream& operator<< ( ostream& os, jInstanceRecord& jIR );  //Output
  jInstanceRecord(const jInstanceRecord& rhs);  //Copy Constructor
  virtual jInstanceRecord& operator=(const jInstanceRecord& rhs);  //Assignment
  virtual jInstanceRecord& operator+=(const jInstanceRecord& rhs);
  virtual jInstanceRecord& operator-=(const jInstanceRecord& rhs);
  const jClassRecord* getClass(void) const  //Get class owner
  {
    return m_pClass;
  }
  void add(const JVMPI_Event* event, JVMPI_CallTrace* pTrace);  //Process new allocation event
  void sub(void);                                               //Process free event
  void setSize(jlong size)                                      //Set object size
  {
    m_Size=size;
  }
  virtual BOOL operator<(const jBaseRecord& rhs) const  //Operator less than
  {
      return (m_MemoryCount<((jInstanceRecord&)rhs).m_MemoryCount);
  }
  virtual BOOL operator>(const jBaseRecord& rhs) const  //Operator greater than
  {
      return (m_MemoryCount>((jInstanceRecord&)rhs).m_MemoryCount);
  }
  void setShowAllocSites(const BOOL bNewValue)
  {
    m_bShowAllocSites=bNewValue;
  }
  BOOL getShowAllocSites(void) const
  {
    return m_bShowAllocSites;
  }
};

