/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  See header file.
*
*  SCCS: jMethodRecord.cpp @(#)jMethodRecord.cpp	1.1 05/28/99 
*
***/

#include <fstream.h>  //Needed for definitions of streams etc...
#include <jvmpi.h>    //Needed for definitions of jobjectID etc...
#ifdef WIN32
#include <windows.h>
#include <process.h>
#else
#include <strings.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#endif
#include "utility.h"  //Needed for time utilities, 
#include "hash.h"
#include "jBaseRecord.h"
#include "jInstanceRecord.h"
#include "jClassRecord.h"
#include "jGCRecord.h"
#include "jMethodEvent.h"
#include "jMethodInvokedBy.h"
#include "jMethodRecord.h"
#include "jThreadRecord.h"
#include "jProf.h"

//Memory management functions
void manageInvoker(void* pValue);

jMethodRecord::jMethodRecord(jmethodID id) : m_ID(id), m_InvCount(0), m_ExeTime(0), m_CPUTime(0), m_MaxCPUTime(0), m_MinCPUTime(0), m_InvokerHash(20, manageInvoker)
{
}

jMethodRecord::jMethodRecord(const jMethodRecord& rhs) : m_InvokerHash(rhs.m_InvokerHash)
{
  m_ID=rhs.m_ID;
  //m_InvokerHash=rhs.m_InvokerHash;
  m_InvCount=rhs.m_InvCount;
  m_ExeTime=rhs.m_ExeTime;
  m_CPUTime=rhs.m_CPUTime;
  m_MaxCPUTime=rhs.m_MaxCPUTime;
  m_MinCPUTime=rhs.m_MinCPUTime;
}

jMethodRecord& jMethodRecord::operator=(jMethodRecord& rhs)
{
  if(this==&rhs)
    return *this;
  
  m_ID=rhs.m_ID;
  m_InvokerHash=rhs.m_InvokerHash;
  m_InvCount=rhs.m_InvCount;
  m_ExeTime=rhs.m_ExeTime;
  m_CPUTime=rhs.m_CPUTime;
  m_MaxCPUTime=rhs.m_MaxCPUTime;
  m_MinCPUTime=rhs.m_MinCPUTime;
  return *this;
}

jMethodRecord::~jMethodRecord()
{
}

jMethodRecord& jMethodRecord::operator+=(jMethodRecord& rhs)
{
  //Got to deal with invoker hash
  hashIterator mthdIterator(rhs.m_InvokerHash);
  for(jMethodInvokedBy* pMIB=(jMethodInvokedBy*)mthdIterator.getValue();pMIB!=NULL;pMIB=(jMethodInvokedBy*)mthdIterator.getValue())
  {
    jMethodInvokedBy* pInvoker=(jMethodInvokedBy*)m_InvokerHash.getValue(pMIB->getID());
    if(pInvoker==NULL)
    {
      pInvoker=new jMethodInvokedBy(*pMIB);
      m_InvokerHash.insert(pInvoker->getID(),pInvoker);
    }
    else
    {
      *pInvoker+=*pMIB;
    }
  }
  m_InvCount+=rhs.m_InvCount;
  m_ExeTime+=rhs.m_ExeTime;
  m_CPUTime+=rhs.m_CPUTime;
  if(rhs.m_MaxCPUTime>m_MaxCPUTime)
    m_MaxCPUTime=rhs.m_MaxCPUTime;
  if(rhs.m_MinCPUTime<m_MinCPUTime)
    m_MinCPUTime=rhs.m_MinCPUTime;

  return *this;
}

jMethodInvokedBy* jMethodRecord::getInvoker(jmethodID id)
{
  jMethodInvokedBy* pInvoker=(jMethodInvokedBy*)m_InvokerHash.getValue(id);
  if(pInvoker==NULL)
  {
    pInvoker=new jMethodInvokedBy();
    pInvoker->setID(id);
    m_InvokerHash.insert(id, pInvoker);
  }
  return pInvoker;
}
void jMethodRecord::update(jMethodEvent* pEvent)
{
  //Use the id from the event to incremement a structure which lists
  //Ids and number of times invoked by
  jMethodEvent* pPrev=pEvent->getPrev();
  jMethodInvokedBy* pInvoker;
  if(pPrev!=NULL)
  {
    pInvoker=getInvoker(pPrev->getID());
    pInvoker->incrementCount();
  }
  m_InvCount++;
  m_ExeTime+=pEvent->getDuration();
  m_CPUTime+=pEvent->getCPUTime();
  if(pEvent->getCPUTime()>m_MaxCPUTime)
    m_MaxCPUTime=pEvent->getCPUTime();
  if(pEvent->getCPUTime()<m_MinCPUTime)
    m_MinCPUTime=pEvent->getCPUTime();
}

ostream& operator<< ( ostream& os, jMethodRecord & jMR )
{
//   os<<"<method_details invoke_count=\""<<jMR.m_InvCount;
//   os<<"\" CPUTime=\""<<(long)jMR.m_CPUTime;
//   os<<"\" ExecutionTime=\""<<(long)jMR.m_ExeTime<<"\">"<<endl;
//   return os<<"</method_details>"<<endl;
  hashIterator mthdIterator(jMR.m_InvokerHash);
  for(jMethodInvokedBy* pMIB=(jMethodInvokedBy*)mthdIterator.getValue();pMIB!=NULL;pMIB=(jMethodInvokedBy*)mthdIterator.getValue())
  	os<<*pMIB;
  return os;
}

void manageInvoker(void* pValue)
{
  delete (jMethodInvokedBy*)pValue;
}
