/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  This class captures information on method utilisation.
*
*  SCCS: jMethodRecord.h @(#)jMethodRecord.h	1.1 05/28/99 
*
***/

class jMethodRecord
{
private:
  jmethodID m_ID;                    //ID
  unsigned long m_InvCount;          //Invocation Count
  unsigned long m_ExeTime;           //Elapsed (Wall) Time(ms)
  jlong         m_CPUTime;           //Total CPU Time(ms)
  jlong         m_MaxCPUTime;        //Maximum CPU Time(ms)
  jlong         m_MinCPUTime;        //Minimum CPU Time(ms)
  hash m_InvokerHash;                //Hash of invokers
public:
  jMethodRecord(jmethodID id);               //Main Constructor
  jMethodRecord(const jMethodRecord& rhs);   //Copy Constructor
  ~jMethodRecord();                          //Destructor
  friend ostream& operator<< ( ostream& os, jMethodRecord& jMR );  //Output
  const jmethodID getID(void) const          //Get ID
  {
    return m_ID;
  }
  void update(jMethodEvent* pEvent);         //Update with Event details
  unsigned long getExeTime(void) const       //Get Elapsed Time
  {
    return m_ExeTime;
  }
  jlong getCPUTime(void) const               //Get CPU Time
  {
    return m_CPUTime;
  }
  jlong getMaxCPUTime(void) const            //Get Maximum CPU Time
  {
    return m_MaxCPUTime;
  }
  jlong getMinCPUTime(void) const            //Get Minimum CPU Time
  {
    return m_MinCPUTime;
  }
  jlong getAvgCPUTime(void) const            //Get Average CPU Time
  {
    return m_CPUTime/m_InvCount;
  }
  unsigned long getInvCount(void) const      //Get Invocation Count
  {
    return m_InvCount;
  }
  jMethodInvokedBy* getInvoker(jmethodID id);  //Get Invoker
  virtual BOOL operator<(const jMethodRecord& other) const  //Operator less than
  {
      return (getCPUTime()<other.getCPUTime());
  }
  virtual BOOL operator>(const jMethodRecord& other) const  //Operator greater
  {
      return (getCPUTime()>other.getCPUTime());
  }
  virtual jMethodRecord& operator=(jMethodRecord& rhs);  //Assignment operator
  virtual jMethodRecord& operator+=(jMethodRecord& rhs); //Add/Assignment
};
