/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  This class is the main profiling class. It makes use of the other classes
*  to maintain and provide information on the current profiling run. Most
*  of it's functionality is provided by the jThreadRecord class.
*
*  SCCS: jProf.h @(#)jProf.h	1.1 06/02/99 
*
***/

//Forward Declaration
class jProf;

//JVMPI Entry Point declaration
#ifdef __cplusplus
extern "C" {
#endif
  JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM *jvm, char *options, void *reserved);
  jProf* theProfiler(jProf* pProfiler=NULL);
#ifdef __cplusplus
}
#endif

//typedef jobject (* LPFNDLLFUNC1)(jobjectID);
//typedef jobjectID (* LPFNDLLFUNC2)(jobject);

class jProf
{
 private:
  short                   m_Version;    //Version of jProf
  hash                    m_ThrHash;    //Hash of live threads
  hash                    m_ClaHash;    //Hash of loaded classes
  hash                    m_GCHash;     //Hash of GC Events
  hash                    m_MthdRecord; //Hash of method use information
  hash                    m_MthdHash;   //Hash of loaded methods
  unsigned long           m_tStart;     //Profiling start timestamp(ms)
  jThreadRecord**         m_pThrHashHistory;  //List of dead threads
  unsigned long           m_ThrHashHistorys;  //Count of dead threads
  JVMPI_RawMonitor        m_Lock;       //Synchronisation member
  char*                   m_pOptionTable[10][3];  //Option table
  ofstream                m_FDump;      //Output file stream
  unsigned long           m_GCKey;      //Value of current GC ID
  JVMPI_Interface*  m_pJVMPI;           //Interface into JVM
  jProf(const jProf& rhs);
  jProf& operator=(const jProf& rhs);
 public:
  jProf(JVMPI_Interface* pJVMPI);      //Main Constructor
  virtual ~jProf(void);                //Destructor
  void setNotify(void);                //Set Notify Function
  unsigned long getDuration(void);     //Get profiling duration(ms)
  friend ostream& operator<< ( ostream& os, jProf& jP );  //Output
  void    addThread(jThreadRecord* pThr);  //Add a thread to live thread hash
  void    delThread(jThreadRecord* pThr);  //Move thread from live to dead
  jThreadRecord*    getThread(void* pKey);  //Get a live thread
  unsigned long    getThreadCount(void)    //Get live thread count (unused)
    {
    return m_ThrHash.getCount();
    }
  //JVMPI specific stuff
  //Wrapper functions for accessing the JVMPI API through this jProf's
  //JVMPI interface
  friend JNIEXPORT void  NotifyEvent(JVMPI_Event* event);
  inline void SuspendThread(const JNIEnv* env);
  inline jint EnableEvent(jint event_type, void *arg);
  inline jint DisableEvent(jint event_type, void *arg);
  void EnableGC(void);//Needs external linkage
  void DisableGC(void);//Needs external linkage
  inline void SetThreadLocalStorage(JNIEnv *env_id, void *ptr);
  inline void* GetThreadLocalStorage(JNIEnv *env_id);
  inline jobjectID GetThreadObject(JNIEnv *env);
  jlong GetCurrentThreadCpuTime(void);//Needs external linkage
  inline jint RequestEvent(jint event_type, void *arg);
  JVMPI_RawMonitor RawMonitorCreate(char* lock_name);
  void RawMonitorDestroy(JVMPI_RawMonitor lock_id);
  void RawMonitorEnter(JVMPI_RawMonitor lock_id);
  void RawMonitorExit(JVMPI_RawMonitor lock_id);
  void RawMonitorNotifyAll(JVMPI_RawMonitor lock_id);
  void RawMonitorWait(JVMPI_RawMonitor lock_id, jlong ms);
  inline void GetCallTrace(JVMPI_CallTrace* pTrace, jint depth);
  void ProfilerExit(jint err_code);//Needs external linkage
  inline jint GetThreadStatus(const JNIEnv* env);
  //Profiling functions
  jClassRecord*  getClassByObject(jThreadRecord* pThr, jobjectID id);  //Get classRecord using a supplied class id
  void delObjectHistory(void);  //Delete accumulated instance history
  inline void getLock(void);           //Acquire jProf lock
  inline void releaseLock(void);       //Release jProf lock
  inline void waitLock(void);          //Wait for the jProf lock
  inline void notifyAll(void);         //Notify all threads waiting for the jProf lock
  void addCallTrace(jBaseRecord* pRec, JNIEnv* env);  //Add a call trace
  JVMPI_CallTrace* getCallTrace(const JNIEnv* env);         //Get a call trace
  void setTraceOptions(char* pOptions);  //Set profiling options
  const char* getTraceOption(const char* pOption) const;  //Get a trace option
  const JVMPI_Method* getMethodByID(jmethodID id) const  //Get JVMPI method structure using supplied id
  {
    return (const JVMPI_Method*)m_MthdHash.getValue(id);
  }
  jMethodRecord* getMethodRecord(jmethodID id);  //Get method profiling details using the supplied method id
  void    addMethodRecord(jmethodID id);  //Add a method invocation record for this method id
  void    addClass(jClassRecord* pCla);  //Add a class to the class hash
  void    delClass(jClassRecord* pCla);  //Remove a class from the class hash
  void    updateClass(jClassRecord* pObj, jobjectID newID);  //Update the class id and relocate it in the class hash
  jClassRecord*  getClass(void* pKey);    //Get a class object given it's id
  void    addGC(jGCRecord* pGC);        //Add a GC record to the GC hash
  jGCRecord*  getGC(void);              //Get a GC record from the GC hash
  jClassRecord* getClassByMethodID(jmethodID id) const;  //Get a class using the supplied method id
  void showHelpMessage(void) const;  //Display a help message to std out
  void updateMethodRecord(jMethodRecord* pMR);  //Update method invocation details by merging in the newly supplied record
};

