/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  See header file.
*
*  SCCS: jThreadRecord.cpp @(#)jThreadRecord.cpp	1.4 06/09/99 
*
***/

#include <fstream.h>  //Needed for definitions of streams etc...
#include <jvmpi.h>    //Needed for definitions of jobjectID etc...
#ifndef WIN32
#include <strings.h>
#include <stdlib.h>
#endif
#include <assert.h>
#include "utility.h"  //Needed for time utilities, 
#include "hash.h"
#include "jBaseRecord.h"
#include "jInstanceRecord.h"
#include "jClassRecord.h"
#include "jGCRecord.h"
#include "jMethodEvent.h"
#include "jMethodInvokedBy.h"
#include "jMethodRecord.h"
#include "jThreadRecord.h"
#include "jProf.h"

//Local functions
//Search Functions
int findGC(void * first, void* second);
int findObjectID(void * first, void* second);
//Memory management functions
extern void manageInstance(void* pValue);
extern void manageMethod(void* pValue);
long unsigned jThreadRecord::monName(0);
char jThreadRecord::szMonName[16];

jThreadRecord::jThreadRecord(JVMPI_Event* event, BOOL bProxy, unsigned short samplePeriod) : jBaseRecord(event), m_ID(event->u.thread_start.thread_id), m_ObjHash(200, manageInstance), m_MthdRecord(500, manageMethod)
{
  int strLen=0;

  m_CPUTime=0;
  m_pMonArr=NULL;
  m_pMthdArr=NULL;
  m_OutMthdEventsSize=m_OutMthdEvents=m_MthdEvents=m_MonEvents=0;
  m_GroupName=NULL;
  m_ParentName=NULL;
  m_Name=NULL;
  m_ID=NULL;
  m_pCurrentMethod=NULL;
  m_SamplePeriod=samplePeriod;
  m_SampleStartCount=m_SampleEndCount=0;
  memset(&m_RequestedEvent,0,sizeof(m_RequestedEvent));
  m_Lock=theProfiler()->RawMonitorCreate(jThreadRecord::getNextMonitorName());
  if(bProxy==TRUE)
    return;
  m_ID=event->u.thread_start.thread_id;
  if(event->u.thread_start.group_name)
  {
    m_GroupName=getXMLSafeName(event->u.thread_start.group_name);
//     strLen=strlen(event->u.thread_start.group_name);
//     m_GroupName=new char[strLen+1];
//     strcpy(m_GroupName,event->u.thread_start.group_name);
  }
  else
    m_GroupName=NULL;

  if(event->u.thread_start.parent_name)
  {
    m_ParentName=getXMLSafeName(event->u.thread_start.parent_name);
//     strLen=strlen(event->u.thread_start.parent_name);
//     m_ParentName=new char[strLen+1];
//     strcpy(m_ParentName,event->u.thread_start.parent_name);
  }
  else
    m_ParentName=NULL;

  if(event->u.thread_start.thread_name)
  {
    m_Name=getXMLSafeName(event->u.thread_start.thread_name);
//     strLen=strlen(event->u.thread_start.thread_name);
//     m_Name=new char[strLen+1];
//     strcpy(m_Name,event->u.thread_start.thread_name);
  }
  else
    m_Name=NULL;
}

void  jThreadRecord::delObjectHistory(void)
{
  m_ObjHash.empty();
}

jThreadRecord::~jThreadRecord(void)
{
  if(m_GroupName)
    delete[] m_GroupName;
  if(m_ParentName)
    delete[] m_ParentName;
  if(m_Name)
    delete[] m_Name;
  if(m_pMonArr)
    delete[] m_pMonArr;
  if(m_OutMthdEventsSize>0)
  {
    for(unsigned long i=0;i<m_OutMthdEventsSize;i++)
      delete m_pMthdArr[i];
    delete[] m_pMthdArr;
  }
  theProfiler()->RawMonitorDestroy(m_Lock);
}

ostream& operator<< ( ostream& os, jThreadRecord& jTR )
{
  jBaseRecord& jBR=jTR;

  os<<"<thread "<<jBR;
  os<<" cpu_time=\""<<jTR.m_CPUTime;
  os<<"\" group_name=\"";
  if(jTR.m_GroupName)
    os<<jTR.m_GroupName;
  else
    os<<"none";
  os<<"\" parent_name=\"";
  if(jTR.m_ParentName)
    os<<jTR.m_ParentName;
  else
    os<<"none";
  os<<"\" thread_name=\"";
  if(jTR.m_Name)
    os<<jTR.m_Name;
  else
    os<<"none";
  os<<"\" id=\""<<jTR.m_ID;
  os<<"\" monitor_wait_count=\""<<jTR.m_MonEvents;
  os<<"\" method_call_count=\""<<jTR.m_MthdEvents;
  
  os<<"\" env=\""<<jTR.getEnv()<<"\"";

  if(!STRCMP(theProfiler()->getTraceOption("thr_use"),"y"))
  {
    os<<">"<<endl<<"<wait_list name=\""<<jTR.getName()<<"\">"<<endl;
    if(jTR.getMonEvents()==0)
    {
      os<<"No monitor waits for this thread."<<endl;
    }
    else
    {
      const jMonitorRecord**  ppMR=new const jMonitorRecord*[jTR.getMonEvents()];
      memset(ppMR,0,jTR.getMonEvents()*sizeof(jMonitorRecord*));
      const jMonitorRecord* pMon=jTR.getMonArray();
      for(unsigned long j=0;j<jTR.getMonEvents();j++,pMon++)
      {
        ppMR[j]=pMon;
      }
      //At this stage ppMR contains a series of pointers all pointing at the original pMon records
			quicksort(ppMR,0,jTR.getMonEvents()-1,FALSE);
			/*
      //To do a bubble sort, step through the array of pointers and
      for(unsigned long k=0;k<jTR.getMonEvents()-1;k++)
      {
        for(unsigned long j=jTR.getMonEvents()-1;j>k;j--)
        {
          if(*(ppMR[j-1])<*(ppMR[j]))
          {
            const jMonitorRecord* pTmp=ppMR[j-1];
            ppMR[j-1]=ppMR[j];
            ppMR[j]=pTmp;
          }
        }
      }
			*/
      //Now have a sorted list for the thread
#ifdef WIN32
      for(j=0;j<jTR.getMonEvents();j++)
#else
      for(unsigned long j=0;j<jTR.getMonEvents();j++)
#endif
      {
        os<<*(jMonitorRecord*)ppMR[j];
      }
      delete[] ppMR;
    }
    os<<"</wait_list>"<<endl;
    return os<<"</thread>"<<endl;
  }
  else
    return os<<"/>"<<endl;
}

jInstanceRecord* jThreadRecord::addObjectRecord(JVMPI_Event* event, jClassRecord* pClass)
{
  //When adding an object, just update the object record maintained
  //by the thread Firstly try to get an object record for this class
  jInstanceRecord* pObj=getObjectRecord(pClass);
  if(pObj==NULL)
  {
    pObj=new jInstanceRecord(event, pClass, theProfiler()->getCallTrace(getEnv()));
    if(FAIL==m_ObjHash.insert(pClass, pObj)) //Hash the pointer to the class record
    {
      cout<<"jThreadRecord: Object insert failed\n";
      exit(-1);
    }
    pObj->setSize(event->u.obj_alloc.size);
  }
  else  //Update the existing record
  {
    pObj->add(event, theProfiler()->getCallTrace(getEnv()));
  }
  return pObj;
}

jInstanceRecord*  jThreadRecord::getObjectRecord(void* pKey)
{
  return (jInstanceRecord*)m_ObjHash.getValue(pKey);
}

void jThreadRecord::setEndDetails(JVMPI_Event* event)
{
  jBaseRecord::setEndDetails();
  m_CPUTime=(unsigned long)theProfiler()->GetCurrentThreadCpuTime();
}

void jThreadRecord::addMonitorEvent(JVMPI_Event* event)
{
  if(m_MonEvents%50==0)
  {
    jMonitorRecord*  pNewEvents=new jMonitorRecord[m_MonEvents+50];
    if(m_MonEvents>0)
    {
      memcpy(pNewEvents,m_pMonArr,(m_MonEvents)*sizeof(jMonitorRecord));
      delete[] m_pMonArr;
    }
    m_pMonArr=pNewEvents;
  }
  //Set the Start Details
  m_pMonArr[m_MonEvents].setStartDetails();
  m_pMonArr[m_MonEvents].setID(event->u.monitor.object);
  //Request the profiler fill in the class details for us;
  jClassRecord* pClass=theProfiler()->getClassByObject(this, event->u.monitor.object);
  m_pMonArr[m_MonEvents].setClass(pClass);
  m_MonEvents++;
}

void jThreadRecord::updateMonitorEvent(JVMPI_Event* event)
{
  m_pMonArr[m_MonEvents-1].setEndDetails();
}

jMethodEvent** jThreadRecord::getMethodEvent(void)
{
  //Allocate method trace blocks in blocks of a 50, that should be ample
  if(m_OutMthdEvents==m_OutMthdEventsSize)
  {
    m_OutMthdEventsSize+=50;
    jMethodEvent**  pNewEvents=new jMethodEvent*[m_OutMthdEventsSize];
    memset(pNewEvents,0,m_OutMthdEventsSize*sizeof(jMethodEvent*));
    if(m_MthdEvents>0)
    {
      for(unsigned long i=0;i<m_OutMthdEvents;i++)
        pNewEvents[i]=m_pMthdArr[i];
      delete[] m_pMthdArr;
    }
    m_pMthdArr=pNewEvents;
  }
  return &m_pMthdArr[m_OutMthdEvents];
}

jMethodEvent* jThreadRecord::getMethodEvent(jmethodID id) const
{
  jMethodEvent* pTmp;
  if(m_OutMthdEvents<1)
    return NULL;
  pTmp=m_pMthdArr[m_OutMthdEvents-1];
  m_pMthdArr[m_OutMthdEvents-1]=NULL;
  return pTmp;

}

void jThreadRecord::addMethodEvent(JVMPI_Event* event)
{
  jMethodEvent* pPrev=m_pCurrentMethod;
  jlong thrTime=theProfiler()->GetCurrentThreadCpuTime();
  if(m_pCurrentMethod)
  {
    m_pCurrentMethod->addCPUTime(thrTime);
  }
  jMethodEvent** pEvent=getMethodEvent();
  if(pEvent==NULL)
  {
    cout<<"jThreadRecord: Could not allocate an event block, terminating..."<<endl;
    theProfiler()->ProfilerExit(-1);
  }
  m_pCurrentMethod=new jMethodEvent(event);
  m_pCurrentMethod->setCPUStamp(thrTime); //Stamp a method to check it's time as soon as it is created
  m_pCurrentMethod->setPrev(pPrev);
  *pEvent=m_pCurrentMethod;
  //Increment counters
  m_MthdEvents++;
  m_OutMthdEvents++;
}

void jThreadRecord::updateMethodEvent(JVMPI_Event* event)
{
  jlong thrTime=theProfiler()->GetCurrentThreadCpuTime();
  //Need to search backwards to find the method that we are exiting and set
  //the details appropriately
  jMethodEvent* pEvent=getMethodEvent(event->u.method.method_id);
  if(pEvent==NULL)//This can only happen when the profiler is terminating
  {
    return;
  }
  pEvent->setEndDetails();
  //Update the jMethodRecord with the information for this event
  //Access the profiler and start to update the information stored
  //in each class file
  pEvent->addCPUTime(thrTime);
  updateMethodRecord(pEvent);
  //The CPU logic is broken if pEvent==m_pCurrentMethod
  //This can only happen if the m_pCurrentMethods previous pointer is set to point
  //at itself!
  m_pCurrentMethod=m_pCurrentMethod->getPrev();
  if(m_pCurrentMethod)
    m_pCurrentMethod->setCPUStamp(thrTime); //Stamp a method to check it's time as soon as it is created
  delete pEvent;
  m_OutMthdEvents--;
}

//This isn't really an assignment, so I shouldn't really use this operator
//Oh well, what the heck!! The commented out code represents what a real
//assignment operator would need to do. Since this code only exists to
//work around the JVMPI thread notification feature (bug) I'm not losing
//too much sleep over it.
jThreadRecord& jThreadRecord::operator=(const jThreadRecord& rhs)
{
  if(this==&rhs)
    return *this;

  jBaseRecord::operator=(rhs);
  if(m_GroupName)
    delete[] m_GroupName;
  if(m_ParentName)
    delete[] m_ParentName;
  if(m_Name)
    delete[] m_Name;

  if(rhs.m_GroupName)
  {
    m_GroupName=new char[strlen(rhs.m_GroupName)+1];
    strcpy(m_GroupName, rhs.m_GroupName);
  }
  if(rhs.m_ParentName)
  {
    m_ParentName=new char[strlen(rhs.m_ParentName)+1];
    strcpy(m_ParentName, rhs.m_ParentName);
  }
  if(rhs.m_Name)
  {
    m_Name=new char[strlen(rhs.m_Name)+1];
    strcpy(m_Name, rhs.m_Name);
  }
  m_ID=rhs.getID();
  memset(&m_RequestedEvent,0,sizeof(m_RequestedEvent));
  setRequestedEvent((JVMPI_Event*)&rhs.m_RequestedEvent);

/*
  m_SamplePeriod=rhs.m_SamplePeriod;
  m_SampleStartCount=rhs.m_SampleStartCount;
  m_SampleEndCount=rhs.m_SampleEndCount;
  if(m_MonEvents>0)
  {
    if(m_pMonArr!=NULL)
      delete[] m_pMonArr;
    m_pMonArr=new jMonitorRecord[m_MonEvents];
    memcpy(m_pMonArr,rhs.m_pMonArr,m_MonEvents*sizeof(jMonitorRecord));
  }
  m_MthdEvents=rhs.m_MthdEvents;
  m_OutMthdEvents=rhs.m_OutMthdEvents;
  m_OutMthdEventsSize=rhs.m_OutMthdEventsSize;
  if(m_OutMthdEvents>0)
  {
    if(m_pMthdArr!=NULL)
    {
      for(unsigned long i=0;i<m_OutMthdEvents;i++)
      {
        delete m_pMthdArr[i];
      }
    }
    m_pMthdArr=new jMethodEvent*[m_OutMthdEventsSize];
    for(unsigned long i=0;i<m_OutMthdEvents;i++)
    {
      m_pMthdArr[i]=rhs.m_pMthdArr[i];
    }
  }
  m_CPUTime=rhs.m_CPUTime;
  m_MonEvents=rhs.m_MonEvents;
  if(m_MonEvents>0)
  {
    delete[] m_pMonArr;
    m_pMonArr=new jMonitorRecord[m_MonEvents];
    memcpy(m_pMonArr,rhs.m_pMonArr,m_MonEvents*sizeof(jMonitorRecord));
  }
  m_ObjHash=rhs.m_ObjHash;
  m_MthdRecord=rhs.m_MthdRecord;
*/
  return *this;
}

void jThreadRecord::updateMethodRecord(jMethodEvent * pEvent)
{
  jMethodRecord* pRecord=getMethodRecord(pEvent->getID());
  if(pRecord!=NULL)
    pRecord->update(pEvent);
}

void jThreadRecord::ModifyEventTimeByGC(jlong t)
{
  //Ignore the effect on wall time and only reduce the time for
  //the currently active event
  if(m_pCurrentMethod)
  {
    m_pCurrentMethod->subCPUTime(t);
  }
}

void jThreadRecord::archiveInstanceData(void)
{
  //When this method is called copy the current
  //state of the instance allocation records into
  //an archive.
}

jMethodRecord* jThreadRecord::getMethodRecord(jmethodID id)
{
  jMethodRecord* pRecord=(jMethodRecord*)m_MthdRecord.getValue(id);
  if(pRecord==NULL)//No data entered for this method yet, create a record and allocate
  {
    pRecord=new jMethodRecord(id);
    m_MthdRecord.insert(id, pRecord);
  }
  return pRecord;
}
BOOL jThreadRecord::doStartSample(void)
{
  if(++m_SampleStartCount==m_SamplePeriod)
  {
    m_SampleStartCount=0;
    return TRUE;
  }
  else
    return FALSE;
}
BOOL jThreadRecord::doEndSample(void)
{
  if(++m_SampleEndCount==m_SamplePeriod)
  {
    m_SampleEndCount=0;
    return TRUE;
  }
  else
    return FALSE;
}

void  jThreadRecord::getLock(void)
{
  theProfiler()->RawMonitorEnter(m_Lock);
}

void  jThreadRecord::notifyAll(void)
{
  theProfiler()->RawMonitorNotifyAll(m_Lock);
}

void  jThreadRecord::releaseLock(void)
{
	theProfiler()->RawMonitorExit(m_Lock);
	notifyAll();
}

int findGC(void * first, void* second)
{
  unsigned long f=*(unsigned long*)first;
  unsigned long s=*(unsigned long*)second;
  if (f<s)
    return -1;
  else
  {
    if (f==s)
      return 0;
    else
      return 1;
  }
}

int findObjectID(void * first, void* second)
{
  jobjectID f=*(jobjectID*)first;
  jobjectID s=*(jobjectID*)second;
  if (f<s)
    return -1;
  else
  {
    if (f==s)
      return 0;
    else
      return 1;
  }
}

ostream& operator<< ( ostream& os, jMonitorRecord& jMR )
{
  jBaseRecord& jBR=jMR;
  
  os<<"<monitor_wait "<<jBR;
  os<<" duration=\""<<jMR.getDuration();
  if(jMR.getClass())
  {
    if(jMR.getClass()->getName())
      os<<"\" target=\""<<jMR.getClass()->getName();
    else
      os<<"\" target=\"UNIDENTIFIED\"";
  }
  else
  {
    os<<" target=\"UNIDENTIFIED\"";
  }
  return os<<"\"/>"<<endl;
  
}
