/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  This class implements some simple information relating to monitor
*  activity for a thread.
*
*  SCCS: jThreadRecord.h @(#)jThreadRecord.h	1.1 06/02/99 
*
***/

class jMonitorRecord : public jBaseRecord
{
private:
  jobjectID m_ID;                          //ID
  jClassRecord* m_pCla;                    //Class of synchronised object
  jMonitorRecord(const jMonitorRecord& rhs);
  jMonitorRecord& operator=(const jMonitorRecord& rhs);
public:
  jMonitorRecord(){};                        //Main Constructor
  friend ostream& operator<< ( ostream& os, jMonitorRecord& jMR );  //Output
  void setClass(jClassRecord* pCla)        //Set Class
  {
    m_pCla=pCla;
  }
  const jClassRecord* getClass(void) const //Get Class
  {
    return m_pCla;
  }
  void setID(jobjectID id)                 //Set ID
  {
    m_ID=id;
  }
  const jobjectID getID(void) const        //Get ID
  {
    return m_ID;
  }
  virtual BOOL operator<(const jBaseRecord& rhs) const  //Operator less than
  {
      return (getDuration()<((jMonitorRecord&)rhs).getDuration());
  }
  virtual BOOL operator>(const jBaseRecord& rhs) const  //Operator greater than
  {
      return (getDuration()>((jMonitorRecord&)rhs).getDuration());
  }
};

/***
*
*  Description
*  This class implements all functions required to manage threads. Most
*  data is held on a per thread basis to minimise synchronisation overhead
*  in the profiler.
*
***/
class jThreadRecord : public jBaseRecord
{
private:
  jobjectID        m_ID;                  //ID
  hash             m_ObjHash;             //Hash of objects allocated
  hash             m_MthdRecord;          //Hash of methods used
  unsigned long    m_CPUTime;             //CPU time consumed 
  unsigned long    m_MonEvents;           //Monitor synchronisations count
  unsigned long    m_MthdEvents;          //Method call count
  unsigned long    m_OutMthdEvents;       //Outstanding method calls
  unsigned long    m_OutMthdEventsSize;   //Size of outstanding methods array
  jMonitorRecord*  m_pMonArr;             //Array of monitor records
  jMethodEvent**   m_pMthdArr;            //Array of method calls
  char*            m_GroupName;           //Name of Thread's Group
  char*            m_ParentName;          //Name of Thread's Parent
  char*            m_Name;                //Thread's name
  jMethodEvent*    m_pCurrentMethod;      //Current executing method
  JVMPI_Event      m_RequestedEvent;      //JVMPI requested event data
  unsigned short   m_SamplePeriod;        //Method Sampling Frequency
  unsigned short   m_SampleStartCount;    //Current Sample Count
  unsigned short   m_SampleEndCount;      //Current Sample Count
  JVMPI_RawMonitor m_Lock;                //Synchronisation member
	static char szMonName[16];              //Next monitor name
	static unsigned long monName;           //Used to calculate next monitor name
  jThreadRecord(const jThreadRecord& rhs);
public:
  jThreadRecord(JVMPI_Event* event, BOOL bProxy=FALSE, unsigned short samplePeriod=1); //Main Constructor
  virtual ~jThreadRecord(void);           //Destructor
  friend ostream& operator<< ( ostream& os, jThreadRecord& jP );  //Output
  jInstanceRecord* addObjectRecord(JVMPI_Event* event, jClassRecord* pClass);  //Add the allocation details to the information maintained by this thread 
  jInstanceRecord*  getObjectRecord(void* pKey);  //Get allocation details
  void  delObjectHistory(void);           //Delete allocation details
  void    addMonitorEvent(JVMPI_Event* event);  //Add a monitor event
  void    updateMonitorEvent(JVMPI_Event* event);  //Update MonitorEvent details
  jMethodEvent** getMethodEvent(void);    //Get an empty MethodEvent object
  jMethodEvent* getMethodEvent(jmethodID id) const;  //Get the specified event
  void    addMethodEvent(JVMPI_Event* event);  //Add the event
  void    updateMethodEvent(JVMPI_Event* event);  //Update event details
  virtual void  setEndDetails(JVMPI_Event* event=NULL);  //Set end details
  const jobjectID& getID(void) const      //Get ID
  {
    return m_ID;
  }
  void setID(const jobjectID& ID)         //Set ID
  {
    m_ID=ID;
  }
  const char* getName(void) const         //Get Name
  {
    return m_Name;
  }
  virtual jThreadRecord& operator=(const jThreadRecord& rhs);  //Assignment
  void updateMethodRecord(jMethodEvent * pEvent);  //Update method details
  void ModifyEventTimeByGC(jlong t);      //Remove GC time from method CPU time
  const jMonitorRecord* getMonArray(void) const  //Get a Monitor events 
  {
    return m_pMonArr;
  }
  const unsigned long getMonEvents(void) const    //Get Count of Monitor events
  {
    return m_MonEvents;
  }
  void setRequestedEvent(JVMPI_Event* event)      //Intern requested event
  {
    delete[] m_RequestedEvent.u.object_dump.data;
    memcpy(&m_RequestedEvent,event,sizeof(JVMPI_Event));
    m_RequestedEvent.u.object_dump.data=new char[m_RequestedEvent.u.object_dump.data_len];
    memcpy(m_RequestedEvent.u.object_dump.data,event->u.object_dump.data,m_RequestedEvent.u.object_dump.data_len);
  }
  void setRequestedHeapEvent(JVMPI_Event* event)  //Not Implemented
  {
    return;
  }
  const JVMPI_Event& getRequestedEvent(void)      //Get event
  {
    return m_RequestedEvent;
  }
  void archiveInstanceData(void);                //Not Implemented
  jMethodRecord* getMethodRecord(jmethodID id);  //Get Method record
  const hash& getMethodRecords(void) const       //Get All ethod record
  {
    return m_MthdRecord;
  }
  void dumpMethodRecords(void)                   //Remove all methods
  {
    m_MthdRecord.empty();
  }
  const hash& getObjectRecords(void) const       //Get Object records
  {
    return m_ObjHash;
  }
  BOOL doStartSample(void);                      //Return whether sample required
  BOOL doEndSample(void);                        //Return whether sample required
  void getLock(void);           //Acquire lock
  void releaseLock(void);       //Release lock
  void notifyAll(void);         //Notify all threads waiting for the lock
	static char* getNextMonitorName(void)
	{
		sprintf(jThreadRecord::szMonName,"_jProf%ul",jThreadRecord::monName++);
		return jThreadRecord::szMonName;
	}
};
