/***
* Copyright (C) 1999 Gary Pennington
* 
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Library General Public
* License as published by the Free Software Foundation; either
* version 2 of the License, or (at your option) any later version.
* 
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
* Library General Public License for more details.
* 
* You should have received a copy of the GNU Library General Public
* License along with this library; if not, write to the
* Free Software Foundation, Inc., 59 Temple Place - Suite 330,
* Boston, MA  02111-1307, USA.
* 
* Contact Details:
* The author can be contacted via email at Gary.Pennington@uk.sun.com
* 
* Sun Microsystems Inc., hereby disclaims all copyright interest in
* the library `jProf' (a library for profiling Java programs) written
* by Gary Pennington.
***/
/***
*
*  Description
*  Utilitity functions which don't belong in any class, eg timing functions.
*
*  SCCS: utility.h @(#)utility.h	1.4 06/09/99 
*
***/

/*Platform Specific*/
#ifndef UTILITY_H
#define UTILITY_H

/*Useful Macros*/
#ifdef WIN32
#define STRCMP stricmp
#else
#define STRCMP strcasecmp
#endif

#ifdef WIN32
#include <windows.h>
#ifdef __cplusplus
extern "C" {
#endif
void Win32GetTime(unsigned long& t);
#ifdef __cplusplus
}
#endif
#define GET_TIME(t)	(Win32GetTime(t))
#else
#ifdef LINUX
#ifdef __cplusplus
extern "C" {
#endif
void LinuxGetTime(unsigned long& t);
#ifdef __cplusplus
}
#endif
#define GET_TIME(t)	(LinuxGetTime(t))
#else
#ifdef __cplusplus
extern "C" {
#endif
void SolarisGetTime(unsigned long& t);
#ifdef __cplusplus
}
#endif
#define GET_TIME(t)	(SolarisGetTime(t))
#endif
#endif

#define FAILURE 1
#define SUCCESS 0

#ifndef BOOL
typedef int BOOL;
#endif

#ifndef TRUE
#define TRUE (BOOL)1
#endif

#ifndef FALSE
#define FALSE (BOOL)0
#endif
#ifdef __cplusplus
extern "C" {
#endif
char* getXMLSafeName(const char* pIn);
#ifdef __cplusplus
}
#endif
template <class X> void quicksort(X** ppAlloc, unsigned long p, unsigned long r, BOOL bDir)
{
	if(p<r)
	{
		unsigned long q;
		if(bDir)
			q=partitionAsc(ppAlloc,p,r);
		else
			q=partitionDsc(ppAlloc,p,r);
		quicksort(ppAlloc,p,q,bDir);
		quicksort(ppAlloc,q+1,r,bDir);
	}
}

template <class X> unsigned long partitionAsc(X** ppAlloc, unsigned long p, unsigned long r)
{
	X& x=*(ppAlloc[p]);
	unsigned long i=p-1;
	unsigned long j=r+1;

	while(1)
	{
		do
		{
			j--;
		}
		while(*ppAlloc[j]>x);
		do
		{
			i++;
		}
		while(*ppAlloc[i]<x);
		if(i<j)
		{
			X* pTmp=ppAlloc[i];
			ppAlloc[i]=ppAlloc[j];
			ppAlloc[j]=pTmp;
		}
		else
			return j;
	}
}

template <class X> unsigned long partitionDsc(X** ppAlloc, unsigned long p, unsigned long r)
{
	X& x=*(ppAlloc[p]);
	unsigned long i=p-1;
	unsigned long j=r+1;

	while(1)
	{
		do
		{
			j--;
		}
		while(x>*ppAlloc[j]);
		do
		{
			i++;
		}
		while(x<*ppAlloc[i]);
		if(i<j)
		{
			X* pTmp=ppAlloc[i];
			ppAlloc[i]=ppAlloc[j];
			ppAlloc[j]=pTmp;
		}
		else
			return j;
	}
}

#endif //UTILITY_H
